@@ -1,7 +1,7 @@
 /*
  *  add.c
  *
- *  Copyright (c) 2006-2011 Pacman Development Team <pacman-dev@archlinux.org>
+ *  Copyright (c) 2006-2013 Pacman Development Team <pacman-dev@archlinux.org>
  *  Copyright (c) 2002-2006 by Judd Vinet <jvinet@zeroflux.org>
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -18,19 +18,15 @@
  *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-#include "config.h"
-
 #include <stdlib.h>
 #include <errno.h>
-#include <time.h>
 #include <string.h>
 #include <limits.h>
 #include <fcntl.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <unistd.h>
-#include <inttypes.h> /* int64_t */
-#include <stdint.h> /* intmax_t */
+#include <stdint.h> /* int64_t */
 
 /* libarchive */
 #include <archive.h>
@@ -41,6 +37,7 @@
 #include "alpm.h"
 #include "alpm_list.h"
 #include "handle.h"
+#include "libarchive-compat.h"
 #include "trans.h"
 #include "util.h"
 #include "log.h"
@@ -71,7 +68,7 @@ int SYMEXPORT alpm_add_pkg(alpm_handle_t *handle, alpm_pkg_t *pkg)
 
 	_alpm_log(handle, ALPM_LOG_DEBUG, "adding package '%s'\n", pkgname);
 
-	if(_alpm_pkg_find(trans->add, pkgname)) {
+	if(alpm_pkg_find(trans->add, pkgname)) {
 		RET_ERR(handle, ALPM_ERR_TRANS_DUP_TARGET, -1);
 	}
 
@@ -125,13 +122,26 @@ static int perform_extraction(alpm_handle_t *handle, struct archive *archive,
 	} else if(ret != ARCHIVE_OK) {
 		_alpm_log(handle, ALPM_LOG_ERROR, _("could not extract %s (%s)\n"),
 				origname, archive_error_string(archive));
-		alpm_logaction(handle, "error: could not extract %s (%s)\n",
+		alpm_logaction(handle, ALPM_CALLER_PREFIX,
+				"error: could not extract %s (%s)\n",
 				origname, archive_error_string(archive));
 		return 1;
 	}
 	return 0;
 }
 
+static int try_rename(alpm_handle_t *handle, const char *src, const char *dest)
+{
+	if(rename(src, dest)) {
+		_alpm_log(handle, ALPM_LOG_ERROR, _("could not rename %s to %s (%s)\n"),
+				src, dest, strerror(errno));
+		alpm_logaction(handle, ALPM_CALLER_PREFIX,
+				"error: could not rename %s to %s (%s)\n", src, dest, strerror(errno));
+		return 1;
+	}
+	return 0;
+}
+
 static int extract_single_file(alpm_handle_t *handle, struct archive *archive,
 		struct archive_entry *entry, alpm_pkg_t *newpkg, alpm_pkg_t *oldpkg)
 {
@@ -146,8 +156,6 @@ static int extract_single_file(alpm_handle_t *handle, struct archive *archive,
 	entryname = archive_entry_pathname(entry);
 	entrymode = archive_entry_mode(entry);
 
-	memset(filename, 0, PATH_MAX); /* just to be sure */
-
 	if(strcmp(entryname, ".INSTALL") == 0) {
 		/* the install script goes inside the db */
 		snprintf(filename, PATH_MAX, "%s%s-%s/install",
@@ -158,6 +166,11 @@ static int extract_single_file(alpm_handle_t *handle, struct archive *archive,
 		snprintf(filename, PATH_MAX, "%s%s-%s/changelog",
 				_alpm_db_path(handle->db_local), newpkg->name, newpkg->version);
 		archive_entry_set_perm(entry, 0644);
+	} else if(strcmp(entryname, ".MTREE") == 0) {
+		/* the mtree file goes inside the db */
+		snprintf(filename, PATH_MAX, "%s%s-%s/mtree",
+				_alpm_db_path(handle->db_local), newpkg->name, newpkg->version);
+		archive_entry_set_perm(entry, 0644);
 	} else if(*entryname == '.') {
 		/* for now, ignore all files starting with '.' that haven't
 		 * already been handled (for future possibilities) */
@@ -170,11 +183,12 @@ static int extract_single_file(alpm_handle_t *handle, struct archive *archive,
 	}
 
 	/* if a file is in NoExtract then we never extract it */
-	if(alpm_list_find_str(handle->noextract, entryname)) {
-		_alpm_log(handle, ALPM_LOG_DEBUG, "%s is in NoExtract, skipping extraction\n",
-				entryname);
-		alpm_logaction(handle, "note: %s is in NoExtract, skipping extraction\n",
-				entryname);
+	if(alpm_list_find(handle->noextract, entryname, _alpm_fnmatch)) {
+		_alpm_log(handle, ALPM_LOG_DEBUG, "%s is in NoExtract,"
+				" skipping extraction of %s\n",
+				entryname, filename);
+		alpm_logaction(handle, ALPM_CALLER_PREFIX,
+				"note: %s is in NoExtract, skipping extraction\n", entryname);
 		archive_read_data_skip(archive);
 		return 0;
 	}
@@ -211,20 +225,21 @@ static int extract_single_file(alpm_handle_t *handle, struct archive *archive,
 					/* if filesystem perms are different than pkg perms, warn user */
 					mode_t mask = 07777;
 					_alpm_log(handle, ALPM_LOG_WARNING, _("directory permissions differ on %s\n"
-								"filesystem: %o  package: %o\n"), entryname, lsbuf.st_mode & mask,
+								"filesystem: %o  package: %o\n"), filename, lsbuf.st_mode & mask,
 							entrymode & mask);
-					alpm_logaction(handle, "warning: directory permissions differ on %s\n"
-							"filesystem: %o  package: %o\n", entryname, lsbuf.st_mode & mask,
+					alpm_logaction(handle, ALPM_CALLER_PREFIX,
+							"warning: directory permissions differ on %s\n"
+							"filesystem: %o  package: %o\n", filename, lsbuf.st_mode & mask,
 							entrymode & mask);
 				}
 				_alpm_log(handle, ALPM_LOG_DEBUG, "extract: skipping dir extraction of %s\n",
-						entryname);
+						filename);
 				archive_read_data_skip(archive);
 				return 0;
 			} else {
 				/* case 10/11: trying to overwrite dir with file/symlink, don't allow it */
 				_alpm_log(handle, ALPM_LOG_ERROR, _("extract: not overwriting dir with file %s\n"),
-						entryname);
+						filename);
 				archive_read_data_skip(archive);
 				return 1;
 			}
@@ -233,24 +248,24 @@ static int extract_single_file(alpm_handle_t *handle, struct archive *archive,
 			if(S_ISDIR(sbuf.st_mode)) {
 				/* the symlink on FS is to a directory, so we'll use it */
 				_alpm_log(handle, ALPM_LOG_DEBUG, "extract: skipping symlink overwrite of %s\n",
-						entryname);
+						filename);
 				archive_read_data_skip(archive);
 				return 0;
 			} else {
 				/* this is BAD. symlink was not to a directory */
 				_alpm_log(handle, ALPM_LOG_ERROR, _("extract: symlink %s does not point to dir\n"),
-						entryname);
+						filename);
 				archive_read_data_skip(archive);
 				return 1;
 			}
 		} else if(S_ISREG(lsbuf.st_mode) && S_ISDIR(entrymode)) {
 			/* case 6: trying to overwrite file with dir */
 			_alpm_log(handle, ALPM_LOG_DEBUG, "extract: overwriting file with dir %s\n",
-					entryname);
+					filename);
 		} else if(S_ISREG(entrymode)) {
 			/* case 4,7: */
 			/* if file is in NoUpgrade, don't touch it */
-			if(alpm_list_find_str(handle->noupgrade, entryname)) {
+			if(alpm_list_find(handle->noupgrade, entryname, _alpm_fnmatch)) {
 				notouch = 1;
 			} else {
 				alpm_backup_t *backup;
@@ -282,23 +297,24 @@ static int extract_single_file(alpm_handle_t *handle, struct archive *archive,
 	STRDUP(entryname_orig, entryname, RET_ERR(handle, ALPM_ERR_MEMORY, -1));
 
 	if(needbackup) {
-		char checkfile[PATH_MAX];
+		char *checkfile;
 		char *hash_local = NULL, *hash_pkg = NULL;
-		int ret;
+		size_t len;
 
-		snprintf(checkfile, PATH_MAX, "%s.paccheck", filename);
+		len = strlen(filename) + 10;
+		MALLOC(checkfile, len,
+				errors++; handle->pm_errno = ALPM_ERR_MEMORY; goto needbackup_cleanup);
+		snprintf(checkfile, len, "%s.paccheck", filename);
 
-		ret = perform_extraction(handle, archive, entry, checkfile, entryname_orig);
-		if(ret == 1) {
-			/* error */
-			FREE(entryname_orig);
-			return 1;
+		if(perform_extraction(handle, archive, entry, checkfile, entryname_orig)) {
+			errors++;
+			goto needbackup_cleanup;
 		}
 
 		hash_local = alpm_compute_md5sum(filename);
 		hash_pkg = alpm_compute_md5sum(checkfile);
 
-		/* update the md5 hash in newpkg's backup (it will be the new orginal) */
+		/* update the md5 hash in newpkg's backup (it will be the new original) */
 		alpm_list_t *i;
 		for(i = alpm_pkg_get_backup(newpkg); i; i = i->next) {
 			alpm_backup_t *backup = i->data;
@@ -320,29 +336,27 @@ static int extract_single_file(alpm_handle_t *handle, struct archive *archive,
 			if(hash_local && hash_pkg && strcmp(hash_local, hash_pkg) != 0) {
 				/* looks like we have a local file that has a different hash as the
 				 * file in the package, move it to a .pacorig */
-				char newpath[PATH_MAX];
-				snprintf(newpath, PATH_MAX, "%s.pacorig", filename);
+				char *newpath;
+				size_t newlen = strlen(filename) + 9;
+				MALLOC(newpath, newlen,
+						errors++; handle->pm_errno = ALPM_ERR_MEMORY; goto needbackup_cleanup);
+				snprintf(newpath, newlen, "%s.pacorig", filename);
 
 				/* move the existing file to the "pacorig" */
-				if(rename(filename, newpath)) {
-					_alpm_log(handle, ALPM_LOG_ERROR, _("could not rename %s to %s (%s)\n"),
-							filename, newpath, strerror(errno));
-					alpm_logaction(handle, "error: could not rename %s to %s (%s)\n",
-							filename, newpath, strerror(errno));
+				if(try_rename(handle, filename, newpath)) {
+						errors++;
 					errors++;
 				} else {
 					/* rename the file we extracted to the real name */
-					if(rename(checkfile, filename)) {
-						_alpm_log(handle, ALPM_LOG_ERROR, _("could not rename %s to %s (%s)\n"),
-								checkfile, filename, strerror(errno));
-						alpm_logaction(handle, "error: could not rename %s to %s (%s)\n",
-								checkfile, filename, strerror(errno));
+					if(try_rename(handle, checkfile, filename)) {
 						errors++;
 					} else {
 						_alpm_log(handle, ALPM_LOG_WARNING, _("%s saved as %s\n"), filename, newpath);
-						alpm_logaction(handle, "warning: %s saved as %s\n", filename, newpath);
+						alpm_logaction(handle, ALPM_CALLER_PREFIX,
+								"warning: %s saved as %s\n", filename, newpath);
 					}
 				}
+				free(newpath);
 			} else {
 				/* local file is identical to pkg one, so just remove pkg one */
 				unlink(checkfile);
@@ -356,11 +370,7 @@ static int extract_single_file(alpm_handle_t *handle, struct archive *archive,
 					_alpm_log(handle, ALPM_LOG_DEBUG, "action: installing new file: %s\n",
 							entryname_orig);
 
-					if(rename(checkfile, filename)) {
-						_alpm_log(handle, ALPM_LOG_ERROR, _("could not rename %s to %s (%s)\n"),
-								checkfile, filename, strerror(errno));
-						alpm_logaction(handle, "error: could not rename %s to %s (%s)\n",
-								checkfile, filename, strerror(errno));
+					if(try_rename(handle, checkfile, filename)) {
 						errors++;
 					}
 				} else {
@@ -382,35 +392,37 @@ static int extract_single_file(alpm_handle_t *handle, struct archive *archive,
 				_alpm_log(handle, ALPM_LOG_DEBUG, "action: leaving existing file in place\n");
 				unlink(checkfile);
 			} else {
-				char newpath[PATH_MAX];
+				char *newpath;
+				size_t newlen = strlen(filename) + 8;
 				_alpm_log(handle, ALPM_LOG_DEBUG, "action: keeping current file and installing"
 						" new one with .pacnew ending\n");
-				snprintf(newpath, PATH_MAX, "%s.pacnew", filename);
-				if(rename(checkfile, newpath)) {
-					_alpm_log(handle, ALPM_LOG_ERROR, _("could not install %s as %s (%s)\n"),
-							filename, newpath, strerror(errno));
-					alpm_logaction(handle, "error: could not install %s as %s (%s)\n",
-							filename, newpath, strerror(errno));
+				MALLOC(newpath, newlen,
+						errors++; handle->pm_errno = ALPM_ERR_MEMORY; goto needbackup_cleanup);
+				snprintf(newpath, newlen, "%s.pacnew", filename);
+				if(try_rename(handle, checkfile, newpath)) {
+					errors++;
 				} else {
 					_alpm_log(handle, ALPM_LOG_WARNING, _("%s installed as %s\n"),
 							filename, newpath);
-					alpm_logaction(handle, "warning: %s installed as %s\n",
-							filename, newpath);
+					alpm_logaction(handle, ALPM_CALLER_PREFIX,
+							"warning: %s installed as %s\n", filename, newpath);
 				}
+				free(newpath);
 			}
 		}
 
-		FREE(hash_local);
-		FREE(hash_pkg);
+needbackup_cleanup:
+		free(checkfile);
+		free(hash_local);
+		free(hash_pkg);
 	} else {
-		int ret;
-
 		/* we didn't need a backup */
 		if(notouch) {
 			/* change the path to a .pacnew extension */
 			_alpm_log(handle, ALPM_LOG_DEBUG, "%s is in NoUpgrade -- skipping\n", filename);
 			_alpm_log(handle, ALPM_LOG_WARNING, _("extracting %s as %s.pacnew\n"), filename, filename);
-			alpm_logaction(handle, "warning: extracting %s as %s.pacnew\n", filename, filename);
+			alpm_logaction(handle, ALPM_CALLER_PREFIX,
+					"warning: extracting %s as %s.pacnew\n", filename, filename);
 			strncat(filename, ".pacnew", PATH_MAX - strlen(filename));
 		} else {
 			_alpm_log(handle, ALPM_LOG_DEBUG, "extracting %s\n", filename);
@@ -423,11 +435,11 @@ static int extract_single_file(alpm_handle_t *handle, struct archive *archive,
 			unlink(filename);
 		}
 
-		ret = perform_extraction(handle, archive, entry, filename, entryname_orig);
-		if(ret == 1) {
+		if(perform_extraction(handle, archive, entry, filename, entryname_orig)) {
 			/* error */
-			FREE(entryname_orig);
-			return 1;
+			free(entryname_orig);
+			errors++;
+			return errors;
 		}
 
 		/* calculate an hash if this is in newpkg's backup */
@@ -444,7 +456,7 @@ static int extract_single_file(alpm_handle_t *handle, struct archive *archive,
 			backup->hash = newhash;
 		}
 	}
-	FREE(entryname_orig);
+	free(entryname_orig);
 	return errors;
 }
 
@@ -452,10 +464,13 @@ static int commit_single_pkg(alpm_handle_t *handle, alpm_pkg_t *newpkg,
 		size_t pkg_current, size_t pkg_count)
 {
 	int i, ret = 0, errors = 0;
-	int is_upgrade;
+	int is_upgrade = 0;
 	alpm_pkg_t *oldpkg = NULL;
 	alpm_db_t *db = handle->db_local;
 	alpm_trans_t *trans = handle->trans;
+	alpm_progress_t event = ALPM_PROGRESS_ADD_START;
+	alpm_event_t done = ALPM_EVENT_ADD_DONE, start = ALPM_EVENT_ADD_START;
+	const char *log_msg = "adding";
 	const char *pkgfile;
 
 	ASSERT(trans != NULL, return -1);
@@ -463,6 +478,23 @@ static int commit_single_pkg(alpm_handle_t *handle, alpm_pkg_t *newpkg,
 	/* see if this is an upgrade. if so, remove the old package first */
 	alpm_pkg_t *local = _alpm_db_get_pkgfromcache(db, newpkg->name);
 	if(local) {
+		int cmp = _alpm_pkg_compare_versions(newpkg, local);
+		if(cmp < 0) {
+			log_msg = "downgrading";
+			event = ALPM_PROGRESS_DOWNGRADE_START;
+			start = ALPM_EVENT_DOWNGRADE_START;
+			done = ALPM_EVENT_DOWNGRADE_DONE;
+		} else if(cmp == 0) {
+			log_msg = "reinstalling";
+			event = ALPM_PROGRESS_REINSTALL_START;
+			start = ALPM_EVENT_REINSTALL_START;
+			done = ALPM_EVENT_REINSTALL_DONE;
+		} else {
+			log_msg = "upgrading";
+			event = ALPM_PROGRESS_UPGRADE_START;
+			start = ALPM_EVENT_UPGRADE_START;
+			done = ALPM_EVENT_UPGRADE_DONE;
+		}
 		is_upgrade = 1;
 
 		/* we'll need to save some record for backup checks later */
@@ -473,23 +505,21 @@ static int commit_single_pkg(alpm_handle_t *handle, alpm_pkg_t *newpkg,
 
 		/* copy over the install reason */
 		newpkg->reason = alpm_pkg_get_reason(local);
-
-		EVENT(handle, ALPM_EVENT_UPGRADE_START, newpkg, local);
-	} else {
-		is_upgrade = 0;
-		EVENT(handle, ALPM_EVENT_ADD_START, newpkg, NULL);
 	}
 
+	EVENT(handle, start, newpkg, local);
+
 	pkgfile = newpkg->origin_data.file;
 
 	_alpm_log(handle, ALPM_LOG_DEBUG, "%s package %s-%s\n",
-			is_upgrade ? "upgrading" : "adding", newpkg->name, newpkg->version);
+			log_msg, newpkg->name, newpkg->version);
 		/* pre_install/pre_upgrade scriptlet */
 	if(alpm_pkg_has_scriptlet(newpkg) &&
 			!(trans->flags & ALPM_TRANS_FLAG_NOSCRIPTLET)) {
 		const char *scriptlet_name = is_upgrade ? "pre_upgrade" : "pre_install";
-		_alpm_runscriptlet(handle, pkgfile,
-				scriptlet_name, newpkg->version, NULL, 1);
+
+		_alpm_runscriptlet(handle, pkgfile, scriptlet_name,
+				newpkg->version, oldpkg ? oldpkg->version : NULL, 1);
 	}
 
 	/* we override any pre-set reason if we have alldeps or allexplicit set */
@@ -511,7 +541,8 @@ static int commit_single_pkg(alpm_handle_t *handle, alpm_pkg_t *newpkg,
 	/* prepare directory for database entries so permission are correct after
 	   changelog/install script installation (FS#12263) */
 	if(_alpm_local_db_prepare(db, newpkg)) {
-		alpm_logaction(handle, "error: could not create database entry %s-%s\n",
+		alpm_logaction(handle, ALPM_CALLER_PREFIX,
+				"error: could not create database entry %s-%s\n",
 				newpkg->name, newpkg->version);
 		handle->pm_errno = ALPM_ERR_DB_WRITE;
 		ret = -1;
@@ -521,31 +552,20 @@ static int commit_single_pkg(alpm_handle_t *handle, alpm_pkg_t *newpkg,
 	if(!(trans->flags & ALPM_TRANS_FLAG_DBONLY)) {
 		struct archive *archive;
 		struct archive_entry *entry;
-		int cwdfd;
+		struct stat buf;
+		int fd, cwdfd;
 
 		_alpm_log(handle, ALPM_LOG_DEBUG, "extracting files\n");
 
-		if((archive = archive_read_new()) == NULL) {
-			handle->pm_errno = ALPM_ERR_LIBARCHIVE;
-			ret = -1;
-			goto cleanup;
-		}
-
-		archive_read_support_compression_all(archive);
-		archive_read_support_format_all(archive);
-
-		_alpm_log(handle, ALPM_LOG_DEBUG, "archive: %s\n", pkgfile);
-		if(archive_read_open_filename(archive, pkgfile,
-					ALPM_BUFFER_SIZE) != ARCHIVE_OK) {
-			handle->pm_errno = ALPM_ERR_PKG_OPEN;
+		fd = _alpm_open_archive(db->handle, pkgfile, &buf,
+				&archive, ALPM_ERR_PKG_OPEN);
+		if(fd < 0) {
 			ret = -1;
 			goto cleanup;
 		}
 
 		/* save the cwd so we can restore it later */
-		do {
-			cwdfd = open(".", O_RDONLY);
-		} while(cwdfd == -1 && errno == EINTR);
+		OPEN(cwdfd, ".", O_RDONLY);
 		if(cwdfd < 0) {
 			_alpm_log(handle, ALPM_LOG_ERROR, _("could not get current working directory\n"));
 		}
@@ -554,18 +574,14 @@ static int commit_single_pkg(alpm_handle_t *handle, alpm_pkg_t *newpkg,
 		if(chdir(handle->root) != 0) {
 			_alpm_log(handle, ALPM_LOG_ERROR, _("could not change directory to %s (%s)\n"),
 					handle->root, strerror(errno));
+			_alpm_archive_read_free(archive);
+			CLOSE(fd);
 			ret = -1;
 			goto cleanup;
 		}
 
 		/* call PROGRESS once with 0 percent, as we sort-of skip that here */
-		if(is_upgrade) {
-			PROGRESS(handle, ALPM_PROGRESS_UPGRADE_START,
-					newpkg->name, 0, pkg_count, pkg_current);
-		} else {
-			PROGRESS(handle, ALPM_PROGRESS_ADD_START,
-					newpkg->name, 0, pkg_count, pkg_current);
-		}
+		PROGRESS(handle, event, newpkg->name, 0, pkg_count, pkg_current);
 
 		for(i = 0; archive_read_next_header(archive, &entry) == ARCHIVE_OK; i++) {
 			int percent;
@@ -574,7 +590,7 @@ static int commit_single_pkg(alpm_handle_t *handle, alpm_pkg_t *newpkg,
 				/* Using compressed size for calculations here, as newpkg->isize is not
 				 * exact when it comes to comparing to the ACTUAL uncompressed size
 				 * (missing metadata sizes) */
-				int64_t pos = archive_position_compressed(archive);
+				int64_t pos = _alpm_archive_compressed_ftell(archive);
 				percent = (pos * 100) / newpkg->size;
 				if(percent >= 100) {
 					percent = 100;
@@ -583,18 +599,13 @@ static int commit_single_pkg(alpm_handle_t *handle, alpm_pkg_t *newpkg,
 				percent = 0;
 			}
 
-			if(is_upgrade) {
-				PROGRESS(handle, ALPM_PROGRESS_UPGRADE_START,
-						newpkg->name, percent, pkg_count, pkg_current);
-			} else {
-				PROGRESS(handle, ALPM_PROGRESS_ADD_START,
-						newpkg->name, percent, pkg_count, pkg_current);
-			}
+			PROGRESS(handle, event, newpkg->name, percent, pkg_count, pkg_current);
 
 			/* extract the next file from the archive */
 			errors += extract_single_file(handle, archive, entry, newpkg, oldpkg);
 		}
-		archive_read_finish(archive);
+		_alpm_archive_read_free(archive);
+		CLOSE(fd);
 
 		/* restore the old cwd if we have it */
 		if(cwdfd >= 0) {
@@ -602,7 +613,7 @@ static int commit_single_pkg(alpm_handle_t *handle, alpm_pkg_t *newpkg,
 				_alpm_log(handle, ALPM_LOG_ERROR,
 						_("could not restore working directory (%s)\n"), strerror(errno));
 			}
-			close(cwdfd);
+			CLOSE(cwdfd);
 		}
 
 		if(errors) {
@@ -610,12 +621,14 @@ static int commit_single_pkg(alpm_handle_t *handle, alpm_pkg_t *newpkg,
 			if(is_upgrade) {
 				_alpm_log(handle, ALPM_LOG_ERROR, _("problem occurred while upgrading %s\n"),
 						newpkg->name);
-				alpm_logaction(handle, "error: problem occurred while upgrading %s\n",
+				alpm_logaction(handle, ALPM_CALLER_PREFIX,
+						"error: problem occurred while upgrading %s\n",
 						newpkg->name);
 			} else {
 				_alpm_log(handle, ALPM_LOG_ERROR, _("problem occurred while installing %s\n"),
 						newpkg->name);
-				alpm_logaction(handle, "error: problem occurred while installing %s\n",
+				alpm_logaction(handle, ALPM_CALLER_PREFIX,
+						"error: problem occurred while installing %s\n",
 						newpkg->name);
 			}
 		}
@@ -630,7 +643,8 @@ static int commit_single_pkg(alpm_handle_t *handle, alpm_pkg_t *newpkg,
 	if(_alpm_local_db_write(db, newpkg, INFRQ_ALL)) {
 		_alpm_log(handle, ALPM_LOG_ERROR, _("could not update database entry %s-%s\n"),
 				newpkg->name, newpkg->version);
-		alpm_logaction(handle, "error: could not update database entry %s-%s\n",
+		alpm_logaction(handle, ALPM_CALLER_PREFIX,
+				"error: could not update database entry %s-%s\n",
 				newpkg->name, newpkg->version);
 		handle->pm_errno = ALPM_ERR_DB_WRITE;
 		ret = -1;
@@ -642,13 +656,7 @@ static int commit_single_pkg(alpm_handle_t *handle, alpm_pkg_t *newpkg,
 				newpkg->name);
 	}
 
-	if(is_upgrade) {
-		PROGRESS(handle, ALPM_PROGRESS_UPGRADE_START,
-				newpkg->name, 100, pkg_count, pkg_current);
-	} else {
-		PROGRESS(handle, ALPM_PROGRESS_ADD_START,
-				newpkg->name, 100, pkg_count, pkg_current);
-	}
+	PROGRESS(handle, event, newpkg->name, 100, pkg_count, pkg_current);
 
 	/* run the post-install script if it exists  */
 	if(alpm_pkg_has_scriptlet(newpkg)
@@ -661,11 +669,7 @@ static int commit_single_pkg(alpm_handle_t *handle, alpm_pkg_t *newpkg,
 		free(scriptlet);
 	}
 
-	if(is_upgrade) {
-		EVENT(handle, ALPM_EVENT_UPGRADE_DONE, newpkg, oldpkg);
-	} else {
-		EVENT(handle, ALPM_EVENT_ADD_DONE, newpkg, oldpkg);
-	}
+	EVENT(handle, done, newpkg, oldpkg);
 
 cleanup:
 	_alpm_pkg_free(oldpkg);
