@@ -1,7 +1,7 @@
 /*
  *  add.h
  *
- *  Copyright (c) 2006-2011 Pacman Development Team <pacman-dev@archlinux.org>
+ *  Copyright (c) 2006-2013 Pacman Development Team <pacman-dev@archlinux.org>
  *  Copyright (c) 2002-2006 by Judd Vinet <jvinet@zeroflux.org>
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -24,7 +24,7 @@
 #include "alpm_list.h"
 #include "trans.h"
 
-int _alpm_upgrade_packages(pmtrans_t *trans, pmdb_t *db);
+int _alpm_upgrade_packages(alpm_handle_t *handle);
 
 #endif /* _ALPM_ADD_H */
 
