@@ -1,7 +1,7 @@
 /*
  *  alpm.c
  *
- *  Copyright (c) 2006-2011 Pacman Development Team <pacman-dev@archlinux.org>
+ *  Copyright (c) 2006-2013 Pacman Development Team <pacman-dev@archlinux.org>
  *  Copyright (c) 2002-2006 by Judd Vinet <jvinet@zeroflux.org>
  *  Copyright (c) 2005 by Aurelien Foret <orelien@chez.com>
  *  Copyright (c) 2005 by Christian Hamar <krics@linuxforum.hu>
@@ -21,8 +21,6 @@
  *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-#include "config.h"
-
 #ifdef HAVE_LIBCURL
 #include <curl/curl.h>
 #endif
@@ -39,17 +37,18 @@
  * @{
  */
 
-/** Initializes the library.  This must be called before any other
- * functions are called.
+/** Initializes the library.
+ * Creates handle, connects to database and creates lockfile.
+ * This must be called before any other functions are called.
  * @param root the root path for all filesystem operations
  * @param dbpath the absolute path to the libalpm database
  * @param err an optional variable to hold any error return codes
  * @return a context handle on success, NULL on error, err will be set if provided
  */
 alpm_handle_t SYMEXPORT *alpm_initialize(const char *root, const char *dbpath,
-		enum _alpm_errno_t *err)
+		alpm_errno_t *err)
 {
-	enum _alpm_errno_t myerr;
+	alpm_errno_t myerr;
 	const char *lf = "db.lck";
 	size_t lockfilelen;
 	alpm_handle_t *myhandle = _alpm_handle_new();
@@ -88,7 +87,9 @@ cleanup:
 	return NULL;
 }
 
-/** Release the library.  This should be the last alpm call you make.
+/** Release the library.
+ * Disconnects from the database, removes handle and lockfile
+ * This should be the last alpm call you make.
  * After this returns, handle should be considered invalid and cannot be reused
  * in any way.
  * @param myhandle the context handle
@@ -108,7 +109,7 @@ int SYMEXPORT alpm_release(alpm_handle_t *myhandle)
 		myhandle->db_local = NULL;
 	}
 
-	if(alpm_db_unregister_all(myhandle) == -1) {
+	if(alpm_unregister_all_syncdbs(myhandle) == -1) {
 		ret = -1;
 	}
 
