@@ -1,7 +1,7 @@
 /*
  * alpm.h
  *
- *  Copyright (c) 2006-2011 Pacman Development Team <pacman-dev@archlinux.org>
+ *  Copyright (c) 2006-2013 Pacman Development Team <pacman-dev@archlinux.org>
  *  Copyright (c) 2002-2006 by Judd Vinet <jvinet@zeroflux.org>
  *  Copyright (c) 2005 by Aurelien Foret <orelien@chez.com>
  *  Copyright (c) 2005 by Christian Hamar <krics@linuxforum.hu>
@@ -27,14 +27,16 @@
 extern "C" {
 #endif
 
-#include <sys/types.h> /* for off_t */
-#include <time.h> /* for time_t */
-#include <stdarg.h> /* for va_list */
+#include <stdint.h>    /* int64_t */
+#include <sys/types.h> /* off_t */
+#include <stdarg.h>    /* va_list */
+
+/* libarchive */
+#include <archive.h>
+#include <archive_entry.h>
 
 #include <alpm_list.h>
 
-#define DEPRECATED __attribute__((deprecated))
-
 /*
  * Arch Linux Package Management library
  */
@@ -44,15 +46,14 @@ extern "C" {
  * @{
  */
 
+typedef int64_t alpm_time_t;
+
 /*
  * Enumerations
  * These ones are used in multiple contexts, so are forward-declared.
  */
 
-/**
- * Install reasons.
- * Why the package was installed.
- */
+/** Package install reasons. */
 typedef enum _alpm_pkgreason_t {
 	/** Explicitly requested by the user. */
 	ALPM_PKG_REASON_EXPLICIT = 0,
@@ -60,12 +61,22 @@ typedef enum _alpm_pkgreason_t {
 	ALPM_PKG_REASON_DEPEND = 1
 } alpm_pkgreason_t;
 
+/** Location a package object was loaded from. */
 typedef enum _alpm_pkgfrom_t {
-	PKG_FROM_FILE = 1,
-	PKG_FROM_LOCALDB,
-	PKG_FROM_SYNCDB
+	ALPM_PKG_FROM_FILE = 1,
+	ALPM_PKG_FROM_LOCALDB,
+	ALPM_PKG_FROM_SYNCDB
 } alpm_pkgfrom_t;
 
+/** Method used to validate a package. */
+typedef enum _alpm_pkgvalidation_t {
+	ALPM_PKG_VALIDATION_UNKNOWN = 0,
+	ALPM_PKG_VALIDATION_NONE = (1 << 0),
+	ALPM_PKG_VALIDATION_MD5SUM = (1 << 1),
+	ALPM_PKG_VALIDATION_SHA256SUM = (1 << 2),
+	ALPM_PKG_VALIDATION_SIGNATURE = (1 << 3)
+} alpm_pkgvalidation_t;
+
 /** Types of version constraints in dependency specs. */
 typedef enum _alpm_depmod_t {
   /** No version constraint */
@@ -92,9 +103,7 @@ typedef enum _alpm_fileconflicttype_t {
 	ALPM_FILECONFLICT_FILESYSTEM
 } alpm_fileconflicttype_t;
 
-/**
- * PGP signature verification options
- */
+/** PGP signature verification options */
 typedef enum _alpm_siglevel_t {
 	ALPM_SIG_PACKAGE = (1 << 0),
 	ALPM_SIG_PACKAGE_OPTIONAL = (1 << 1),
@@ -106,12 +115,13 @@ typedef enum _alpm_siglevel_t {
 	ALPM_SIG_DATABASE_MARGINAL_OK = (1 << 12),
 	ALPM_SIG_DATABASE_UNKNOWN_OK = (1 << 13),
 
+	ALPM_SIG_PACKAGE_SET = (1 << 27),
+	ALPM_SIG_PACKAGE_TRUST_SET = (1 << 28),
+
 	ALPM_SIG_USE_DEFAULT = (1 << 31)
 } alpm_siglevel_t;
 
-/**
- * PGP signature verification status return codes
- */
+/** PGP signature verification status return codes */
 typedef enum _alpm_sigstatus_t {
 	ALPM_SIGSTATUS_VALID,
 	ALPM_SIGSTATUS_KEY_EXPIRED,
@@ -121,9 +131,7 @@ typedef enum _alpm_sigstatus_t {
 	ALPM_SIGSTATUS_INVALID
 } alpm_sigstatus_t;
 
-/**
- * PGP signature verification status return codes
- */
+/** PGP signature verification status return codes */
 typedef enum _alpm_sigvalidity_t {
 	ALPM_SIGVALIDITY_FULL,
 	ALPM_SIGVALIDITY_MARGINAL,
@@ -144,6 +152,7 @@ typedef struct __alpm_trans_t alpm_trans_t;
 typedef struct _alpm_depend_t {
 	char *name;
 	char *version;
+	char *desc;
 	unsigned long name_hash;
 	alpm_depmod_t mod;
 } alpm_depend_t;
@@ -152,7 +161,7 @@ typedef struct _alpm_depend_t {
 typedef struct _alpm_depmissing_t {
 	char *target;
 	alpm_depend_t *depend;
-	/* this is used in case of remove dependency error only */
+	/* this is used only in the case of a remove dependency error */
 	char *causingpkg;
 } alpm_depmissing_t;
 
@@ -208,6 +217,7 @@ typedef struct _alpm_file_t {
 typedef struct _alpm_filelist_t {
 	size_t count;
 	alpm_file_t *files;
+	char **resolved_path;
 } alpm_filelist_t;
 
 /** Local package or package file backup entry */
@@ -222,11 +232,15 @@ typedef struct _alpm_pgpkey_t {
 	char *uid;
 	char *name;
 	char *email;
-	time_t created;
-	time_t expires;
+	alpm_time_t created;
+	alpm_time_t expires;
+	unsigned int length;
+	unsigned int revoked;
+	char pubkey_algo;
 } alpm_pgpkey_t;
 
-/** Signature result. Contains the key, status, and validity of a given
+/**
+ * Signature result. Contains the key, status, and validity of a given
  * signature.
  */
 typedef struct _alpm_sigresult_t {
@@ -235,7 +249,8 @@ typedef struct _alpm_sigresult_t {
 	alpm_sigvalidity_t validity;
 } alpm_sigresult_t;
 
-/** Signature list. Contains the number of signatures found and a pointer to an
+/**
+ * Signature list. Contains the number of signatures found and a pointer to an
  * array of results.  The array is of size count.
  */
 typedef struct _alpm_siglist_t {
@@ -247,9 +262,7 @@ typedef struct _alpm_siglist_t {
  * Logging facilities
  */
 
-/**
- * Logging Levels
- */
+/** Logging Levels */
 typedef enum _alpm_loglevel_t {
 	ALPM_LOG_ERROR    = 1,
 	ALPM_LOG_WARNING  = (1 << 1),
@@ -258,9 +271,12 @@ typedef enum _alpm_loglevel_t {
 } alpm_loglevel_t;
 
 typedef void (*alpm_cb_log)(alpm_loglevel_t, const char *, va_list);
-int alpm_logaction(alpm_handle_t *handle, const char *fmt, ...);
 
-/** Events.
+int alpm_logaction(alpm_handle_t *handle, const char *prefix,
+		const char *fmt, ...) __attribute__((format(printf, 3, 4)));
+
+/**
+ * Events.
  * NULL parameters are passed to in all events unless specified otherwise.
  */
 typedef enum _alpm_event_t {
@@ -305,6 +321,24 @@ typedef enum _alpm_event_t {
 	 * to the callback, respectively.
 	 */
 	ALPM_EVENT_UPGRADE_DONE,
+	/** Package will be downgraded.
+	 * A pointer to the downgraded package is passed to the callback.
+	 */
+	ALPM_EVENT_DOWNGRADE_START,
+	/** Package was downgraded.
+	 * A pointer to the new package, and a pointer to the old package is passed
+	 * to the callback, respectively.
+	 */
+	ALPM_EVENT_DOWNGRADE_DONE,
+	/** Package will be reinstalled.
+	 * A pointer to the reinstalled package is passed to the callback.
+	 */
+	ALPM_EVENT_REINSTALL_START,
+	/** Package was reinstalled.
+	 * A pointer to the new package, and a pointer to the old package is passed
+	 * to the callback, respectively.
+	 */
+	ALPM_EVENT_REINSTALL_DONE,
 	/** Target package's integrity will be checked. */
 	ALPM_EVENT_INTEGRITY_START,
 	/** Target package's integrity was checked. */
@@ -342,12 +376,30 @@ typedef enum _alpm_event_t {
 	ALPM_EVENT_DISKSPACE_START,
 	/** Disk space usage was computed for a package */
 	ALPM_EVENT_DISKSPACE_DONE,
+	/** An optdepend for another package is being removed
+	 * The requiring package and its dependency are passed to the callback */
+	ALPM_EVENT_OPTDEP_REQUIRED,
+	/** A configured repository database is missing */
+	ALPM_EVENT_DATABASE_MISSING,
+	/** Checking keys used to create signatures are in keyring. */
+	ALPM_EVENT_KEYRING_START,
+	/** Keyring checking is finished. */
+	ALPM_EVENT_KEYRING_DONE,
+	/** Downloading missing keys into keyring. */
+	ALPM_EVENT_KEY_DOWNLOAD_START,
+	/** Key downloading is finished. */
+	ALPM_EVENT_KEY_DOWNLOAD_DONE
 } alpm_event_t;
 
 /** Event callback */
 typedef void (*alpm_cb_event)(alpm_event_t, void *, void *);
 
-/** Questions */
+/**
+ * Questions.
+ * Unlike the events or progress enumerations, this enum has bitmask values
+ * so a frontend can use a bitmask map to supply preselected answers to the
+ * different types of questions.
+ */
 typedef enum _alpm_question_t {
 	ALPM_QUESTION_INSTALL_IGNOREPKG = 1,
 	ALPM_QUESTION_REPLACE_PKG = (1 << 1),
@@ -366,11 +418,14 @@ typedef void (*alpm_cb_question)(alpm_question_t, void *, void *, void *, int *)
 typedef enum _alpm_progress_t {
 	ALPM_PROGRESS_ADD_START,
 	ALPM_PROGRESS_UPGRADE_START,
+	ALPM_PROGRESS_DOWNGRADE_START,
+	ALPM_PROGRESS_REINSTALL_START,
 	ALPM_PROGRESS_REMOVE_START,
 	ALPM_PROGRESS_CONFLICTS_START,
 	ALPM_PROGRESS_DISKSPACE_START,
 	ALPM_PROGRESS_INTEGRITY_START,
 	ALPM_PROGRESS_LOAD_START,
+	ALPM_PROGRESS_KEYRING_START
 } alpm_progress_t;
 
 /** Progress callback */
@@ -530,8 +585,8 @@ const char *alpm_option_get_arch(alpm_handle_t *handle);
 /** Sets the targeted architecture. */
 int alpm_option_set_arch(alpm_handle_t *handle, const char *arch);
 
-int alpm_option_get_usedelta(alpm_handle_t *handle);
-int alpm_option_set_usedelta(alpm_handle_t *handle, int usedelta);
+double alpm_option_get_deltaratio(alpm_handle_t *handle);
+int alpm_option_set_deltaratio(alpm_handle_t *handle, double ratio);
 
 int alpm_option_get_checkspace(alpm_handle_t *handle);
 int alpm_option_set_checkspace(alpm_handle_t *handle, int checkspace);
@@ -539,6 +594,12 @@ int alpm_option_set_checkspace(alpm_handle_t *handle, int checkspace);
 alpm_siglevel_t alpm_option_get_default_siglevel(alpm_handle_t *handle);
 int alpm_option_set_default_siglevel(alpm_handle_t *handle, alpm_siglevel_t level);
 
+alpm_siglevel_t alpm_option_get_local_file_siglevel(alpm_handle_t *handle);
+int alpm_option_set_local_file_siglevel(alpm_handle_t *handle, alpm_siglevel_t level);
+
+alpm_siglevel_t alpm_option_get_remote_file_siglevel(alpm_handle_t *handle);
+int alpm_option_set_remote_file_siglevel(alpm_handle_t *handle, alpm_siglevel_t level);
+
 /** @} */
 
 /** @addtogroup alpm_api_databases Database Functions
@@ -552,7 +613,7 @@ int alpm_option_set_default_siglevel(alpm_handle_t *handle, alpm_siglevel_t leve
  * libalpm functions.
  * @return a reference to the local database
  */
-alpm_db_t *alpm_option_get_localdb(alpm_handle_t *handle);
+alpm_db_t *alpm_get_localdb(alpm_handle_t *handle);
 
 /** Get the list of sync databases.
  * Returns a list of alpm_db_t structures, one for each registered
@@ -560,7 +621,7 @@ alpm_db_t *alpm_option_get_localdb(alpm_handle_t *handle);
  * @param handle the context handle
  * @return a reference to an internal list of alpm_db_t structures
  */
-alpm_list_t *alpm_option_get_syncdbs(alpm_handle_t *handle);
+alpm_list_t *alpm_get_syncdbs(alpm_handle_t *handle);
 
 /** Register a sync database of packages.
  * @param handle the context handle
@@ -569,21 +630,21 @@ alpm_list_t *alpm_option_get_syncdbs(alpm_handle_t *handle);
  * database; note that this must be a '.sig' file type verification
  * @return an alpm_db_t* on success (the value), NULL on error
  */
-alpm_db_t *alpm_db_register_sync(alpm_handle_t *handle, const char *treename,
+alpm_db_t *alpm_register_syncdb(alpm_handle_t *handle, const char *treename,
 		alpm_siglevel_t level);
 
+/** Unregister all package databases.
+ * @param handle the context handle
+ * @return 0 on success, -1 on error (pm_errno is set accordingly)
+ */
+int alpm_unregister_all_syncdbs(alpm_handle_t *handle);
+
 /** Unregister a package database.
  * @param db pointer to the package database to unregister
  * @return 0 on success, -1 on error (pm_errno is set accordingly)
  */
 int alpm_db_unregister(alpm_db_t *db);
 
-/** Unregister all package databases.
- * @param handle the context handle
- * @return 0 on success, -1 on error (pm_errno is set accordingly)
- */
-int alpm_db_unregister_all(alpm_handle_t *handle);
-
 /** Get the name of a package database.
  * @param db pointer to the package database
  * @return the name of the package database, NULL on error
@@ -615,7 +676,7 @@ int alpm_db_add_server(alpm_db_t *db, const char *url);
 int alpm_db_remove_server(alpm_db_t *db, const char *url);
 /** @} */
 
-int alpm_db_update(int level, alpm_db_t *db);
+int alpm_db_update(int force, alpm_db_t *db);
 
 /** Get a package entry from a package database.
  * @param db pointer to the package database to get the package from
@@ -635,7 +696,7 @@ alpm_list_t *alpm_db_get_pkgcache(alpm_db_t *db);
  * @param name of the group
  * @return the groups entry on success, NULL on error
  */
-alpm_group_t *alpm_db_readgroup(alpm_db_t *db, const char *name);
+alpm_group_t *alpm_db_get_group(alpm_db_t *db, const char *name);
 
 /** Get the group cache of a package database.
  * @param db pointer to the package database to get the group from
@@ -648,16 +709,7 @@ alpm_list_t *alpm_db_get_groupcache(alpm_db_t *db);
  * @param needles a list of regular expressions to search for
  * @return the list of packages matching all regular expressions on success, NULL on error
  */
-alpm_list_t *alpm_db_search(alpm_db_t *db, const alpm_list_t* needles);
-
-/** Set install reason for a package in db.
- * @param handle the context handle
- * @param pkg the package to update
- * @param reason the new install reason
- * @return 0 on success, -1 on error (pm_errno is set accordingly)
- */
-int alpm_db_set_pkgreason(alpm_handle_t *handle, alpm_pkg_t *pkg,
-		alpm_pkgreason_t reason);
+alpm_list_t *alpm_db_search(alpm_db_t *db, const alpm_list_t *needles);
 
 /** @} */
 
@@ -683,6 +735,13 @@ int alpm_db_set_pkgreason(alpm_handle_t *handle, alpm_pkg_t *pkg,
 int alpm_pkg_load(alpm_handle_t *handle, const char *filename, int full,
 		alpm_siglevel_t level, alpm_pkg_t **pkg);
 
+/** Find a package in a list by name.
+ * @param haystack a list of alpm_pkg_t
+ * @param needle the package name
+ * @return a pointer to the package if found or NULL
+ */
+alpm_pkg_t *alpm_pkg_find(alpm_list_t *haystack, const char *needle);
+
 /** Free a package.
  * @param pkg package pointer to free
  * @return 0 on success, -1 on error (pm_errno is set accordingly)
@@ -706,6 +765,14 @@ int alpm_pkg_vercmp(const char *a, const char *b);
  */
 alpm_list_t *alpm_pkg_compute_requiredby(alpm_pkg_t *pkg);
 
+/** Computes the list of packages optionally requiring a given package.
+ * The return value of this function is a newly allocated
+ * list of package names (char*), it should be freed by the caller.
+ * @param pkg a package
+ * @return the list of packages optionally requiring pkg
+ */
+alpm_list_t *alpm_pkg_compute_optionalfor(alpm_pkg_t *pkg);
+
 /** @name Package Property Accessors
  * Any pointer returned by these functions points to internal structures
  * allocated by libalpm. They should not be freed nor modified in any
@@ -754,13 +821,13 @@ const char *alpm_pkg_get_url(alpm_pkg_t *pkg);
  * @param pkg a pointer to package
  * @return the timestamp of the build time
  */
-time_t alpm_pkg_get_builddate(alpm_pkg_t *pkg);
+alpm_time_t alpm_pkg_get_builddate(alpm_pkg_t *pkg);
 
 /** Returns the install timestamp of the package.
  * @param pkg a pointer to package
  * @return the timestamp of the install time
  */
-time_t alpm_pkg_get_installdate(alpm_pkg_t *pkg);
+alpm_time_t alpm_pkg_get_installdate(alpm_pkg_t *pkg);
 
 /** Returns the packager's name.
  * @param pkg a pointer to package
@@ -827,7 +894,7 @@ alpm_list_t *alpm_pkg_get_depends(alpm_pkg_t *pkg);
 
 /** Returns the list of package optional dependencies.
  * @param pkg a pointer to package
- * @return a reference to an internal list of strings.
+ * @return a reference to an internal list of alpm_depend_t structures.
  */
 alpm_list_t *alpm_pkg_get_optdepends(alpm_pkg_t *pkg);
 
@@ -869,7 +936,7 @@ alpm_filelist_t *alpm_pkg_get_files(alpm_pkg_t *pkg);
  * "<filename>\t<md5sum>", where the given md5sum is that of
  * the file as provided by the package.
  * @param pkg a pointer to package
- * @return a reference to an internal list of strings.
+ * @return a reference to a list of alpm_backup_t objects
  */
 alpm_list_t *alpm_pkg_get_backup(alpm_pkg_t *pkg);
 
@@ -887,6 +954,12 @@ alpm_db_t *alpm_pkg_get_db(alpm_pkg_t *pkg);
  */
 const char *alpm_pkg_get_base64_sig(alpm_pkg_t *pkg);
 
+/** Returns the method used to validate a package during install.
+ * @param pkg a pointer to package
+ * @return an enum member giving the validation method
+ */
+alpm_pkgvalidation_t alpm_pkg_get_validation(alpm_pkg_t *pkg);
+
 /* End of alpm_pkg_t accessors */
 /* @} */
 
@@ -913,6 +986,23 @@ size_t alpm_pkg_changelog_read(void *ptr, size_t size,
 
 int alpm_pkg_changelog_close(const alpm_pkg_t *pkg, void *fp);
 
+/** Open a package mtree file for reading.
+ * @param pkg the local package to read the changelog of
+ * @return a archive structure for the package mtree file
+ */
+struct archive *alpm_pkg_mtree_open(alpm_pkg_t *pkg);
+
+/** Read next entry from a package mtree file.
+ * @param pkg the package that the mtree file is being read from
+ * @param archive the archive structure reading from the mtree file
+ * @param entry an archive_entry to store the entry header information
+ * @return 0 if end of archive is reached, non-zero otherwise.
+ */
+int alpm_pkg_mtree_next(const alpm_pkg_t *pkg, struct archive *archive,
+		struct archive_entry **entry);
+
+int alpm_pkg_mtree_close(const alpm_pkg_t *pkg, struct archive *archive);
+
 /** Returns whether the package has an install scriptlet.
  * @return 0 if FALSE, TRUE otherwise
  */
@@ -928,9 +1018,33 @@ off_t alpm_pkg_download_size(alpm_pkg_t *newpkg);
 
 alpm_list_t *alpm_pkg_unused_deltas(alpm_pkg_t *pkg);
 
+/** Set install reason for a package in the local database.
+ * The provided package object must be from the local database or this method
+ * will fail. The write to the local database is performed immediately.
+ * @param pkg the package to update
+ * @param reason the new install reason
+ * @return 0 on success, -1 on error (pm_errno is set accordingly)
+ */
+int alpm_pkg_set_reason(alpm_pkg_t *pkg, alpm_pkgreason_t reason);
+
+
 /* End of alpm_pkg */
 /** @} */
 
+/*
+ * Filelists
+ */
+
+/** Determines whether a package filelist contains a given path.
+ * The provided path should be relative to the install root with no leading
+ * slashes, e.g. "etc/localtime". When searching for directories, the path must
+ * have a trailing slash.
+ * @param filelist a pointer to a package filelist
+ * @param path the path to search for in the package
+ * @return a pointer to the matching file or NULL if not found
+ */
+char *alpm_filelist_contains(alpm_filelist_t *filelist, const char *path);
+
 /*
  * Signatures
  */
@@ -1107,13 +1221,13 @@ char *alpm_dep_compute_string(const alpm_depend_t *dep);
  */
 
 /* checksums */
-char *alpm_compute_md5sum(const char *name);
+char *alpm_compute_md5sum(const char *filename);
 char *alpm_compute_sha256sum(const char *filename);
 
 /** @addtogroup alpm_api_errors Error Codes
  * @{
  */
-enum _alpm_errno_t {
+typedef enum _alpm_errno_t {
 	ALPM_ERR_MEMORY = 1,
 	ALPM_ERR_SYSTEM,
 	ALPM_ERR_BADPERMS,
@@ -1177,19 +1291,19 @@ enum _alpm_errno_t {
 	ALPM_ERR_LIBCURL,
 	ALPM_ERR_EXTERNAL_DOWNLOAD,
 	ALPM_ERR_GPGME
-};
+} alpm_errno_t;
 
 /** Returns the current error code from the handle. */
-enum _alpm_errno_t alpm_errno(alpm_handle_t *handle);
+alpm_errno_t alpm_errno(alpm_handle_t *handle);
 
 /** Returns the string corresponding to an error number. */
-const char *alpm_strerror(enum _alpm_errno_t err);
+const char *alpm_strerror(alpm_errno_t err);
 
 /* End of alpm_api_errors */
 /** @} */
 
 alpm_handle_t *alpm_initialize(const char *root, const char *dbpath,
-		enum _alpm_errno_t *err);
+		alpm_errno_t *err);
 int alpm_release(alpm_handle_t *handle);
 
 enum alpm_caps {
