@@ -1,7 +1,7 @@
 /*
  *  dload.h
  *
- *  Copyright (c) 2006-2011 Pacman Development Team <pacman-dev@archlinux.org>
+ *  Copyright (c) 2006-2013 Pacman Development Team <pacman-dev@archlinux.org>
  *  Copyright (c) 2002-2006 by Judd Vinet <jvinet@zeroflux.org>
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -23,8 +23,6 @@
 #include "alpm_list.h"
 #include "alpm.h"
 
-#include <time.h>
-
 struct dload_payload {
 	alpm_handle_t *handle;
 	const char *tempfile_openmode;
@@ -40,15 +38,17 @@ struct dload_payload {
 	int allow_resume;
 	int errors_ok;
 	int unlink_on_fail;
+	alpm_list_t *servers;
 #ifdef HAVE_LIBCURL
 	CURLcode curlerr;       /* last error produced by curl */
 #endif
+	long respcode;
 };
 
 void _alpm_dload_payload_reset(struct dload_payload *payload);
 
 int _alpm_download(struct dload_payload *payload, const char *localpath,
-		char **final_file);
+		char **final_file, char **final_url);
 
 #endif /* _ALPM_DLOAD_H */
 
