@@ -1,7 +1,7 @@
 /*
  *  error.c
  *
- *  Copyright (c) 2006-2011 Pacman Development Team <pacman-dev@archlinux.org>
+ *  Copyright (c) 2006-2013 Pacman Development Team <pacman-dev@archlinux.org>
  *  Copyright (c) 2002-2006 by Judd Vinet <jvinet@zeroflux.org>
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -18,8 +18,6 @@
  *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-#include "config.h"
-
 #ifdef HAVE_LIBCURL
 #include <curl/curl.h>
 #endif
@@ -29,12 +27,12 @@
 #include "alpm.h"
 #include "handle.h"
 
-enum _alpm_errno_t SYMEXPORT alpm_errno(alpm_handle_t *handle)
+alpm_errno_t SYMEXPORT alpm_errno(alpm_handle_t *handle)
 {
 	return handle->pm_errno;
 }
 
-const char SYMEXPORT *alpm_strerror(enum _alpm_errno_t err)
+const char SYMEXPORT *alpm_strerror(alpm_errno_t err)
 {
 	switch(err) {
 		/* System */
@@ -43,7 +41,7 @@ const char SYMEXPORT *alpm_strerror(enum _alpm_errno_t err)
 		case ALPM_ERR_SYSTEM:
 			return _("unexpected system error");
 		case ALPM_ERR_BADPERMS:
-			return _("insufficient privileges");
+			return _("permission denied");
 		case ALPM_ERR_NOT_A_FILE:
 			return _("could not find or read file");
 		case ALPM_ERR_NOT_A_DIR:
