@@ -1,7 +1,7 @@
 /*
  *  group.h
  *
- *  Copyright (c) 2006-2011 Pacman Development Team <pacman-dev@archlinux.org>
+ *  Copyright (c) 2006-2013 Pacman Development Team <pacman-dev@archlinux.org>
  *  Copyright (c) 2002-2006 by Judd Vinet <jvinet@zeroflux.org>
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -22,15 +22,8 @@
 
 #include "alpm.h"
 
-struct __pmgrp_t {
-	/** group name */
-	char *name;
-	/** list of pmpkg_t packages */
-	alpm_list_t *packages;
-};
-
-pmgrp_t *_alpm_grp_new(const char *name);
-void _alpm_grp_free(pmgrp_t *grp);
+alpm_group_t *_alpm_group_new(const char *name);
+void _alpm_group_free(alpm_group_t *grp);
 
 #endif /* _ALPM_GROUP_H */
 
