@@ -1,7 +1,7 @@
 /*
  *  log.c
  *
- *  Copyright (c) 2006-2011 Pacman Development Team <pacman-dev@archlinux.org>
+ *  Copyright (c) 2006-2013 Pacman Development Team <pacman-dev@archlinux.org>
  *  Copyright (c) 2002-2006 by Judd Vinet <jvinet@zeroflux.org>
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -18,14 +18,9 @@
  *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-#include "config.h"
-
 #include <stdio.h>
 #include <stdarg.h>
-#include <sys/types.h>
-#include <sys/stat.h>
 #include <errno.h>
-#include <time.h>
 
 /* libalpm */
 #include "log.h"
@@ -39,18 +34,18 @@
  */
 
 /** A printf-like function for logging.
+ * @param handle the context handle
+ * @param prefix caller-specific prefix for the log
  * @param fmt output format
  * @return 0 on success, -1 on error (pm_errno is set accordingly)
  */
-int SYMEXPORT alpm_logaction(const char *fmt, ...)
+int SYMEXPORT alpm_logaction(alpm_handle_t *handle, const char *prefix,
+		const char *fmt, ...)
 {
 	int ret;
 	va_list args;
 
-	ALPM_LOG_FUNC;
-
-	/* Sanity checks */
-	ASSERT(handle != NULL, RET_ERR(PM_ERR_HANDLE_NULL, -1));
+	ASSERT(handle != NULL, return -1);
 
 	/* check if the logstream is open already, opening it if needed */
 	if(handle->logstream == NULL) {
@@ -58,47 +53,35 @@ int SYMEXPORT alpm_logaction(const char *fmt, ...)
 		/* if we couldn't open it, we have an issue */
 		if(handle->logstream == NULL) {
 			if(errno == EACCES) {
-				pm_errno = PM_ERR_BADPERMS;
+				handle->pm_errno = ALPM_ERR_BADPERMS;
 			} else if(errno == ENOENT) {
-				pm_errno = PM_ERR_NOT_A_DIR;
+				handle->pm_errno = ALPM_ERR_NOT_A_DIR;
 			} else {
-				pm_errno = PM_ERR_SYSTEM;
+				handle->pm_errno = ALPM_ERR_SYSTEM;
 			}
-		return(-1);
+			return -1;
 		}
 	}
 
 	va_start(args, fmt);
-	ret = _alpm_logaction(handle->usesyslog, handle->logstream, fmt, args);
+	ret = _alpm_logaction(handle, prefix, fmt, args);
 	va_end(args);
 
-	/* TODO	We should add a prefix to log strings depending on who called us.
-	 * If logaction was called by the frontend:
-	 *   USER: <the frontend log>
-	 * and if called internally:
-	 *   ALPM: <the library log>
-	 * Moreover, the frontend should be able to choose its prefix
-	 * (USER by default?):
-	 *   pacman: "PACMAN"
-	 *   kpacman: "KPACMAN"
-	 * This would allow us to share the log file between several frontends
-	 * and know who does what */
-	return(ret);
+	return ret;
 }
 
 /** @} */
 
-void _alpm_log(pmloglevel_t flag, const char *fmt, ...)
+void _alpm_log(alpm_handle_t *handle, alpm_loglevel_t flag, const char *fmt, ...)
 {
 	va_list args;
-	alpm_cb_log logcb = alpm_option_get_logcb();
 
-	if(logcb == NULL) {
+	if(handle == NULL || handle->logcb == NULL) {
 		return;
 	}
 
 	va_start(args, fmt);
-	logcb(flag, fmt, args);
+	handle->logcb(flag, fmt, args);
 	va_end(args);
 }
 
