@@ -1,7 +1,7 @@
 /*
  *  signing.c
  *
- *  Copyright (c) 2008-2012 Pacman Development Team <pacman-dev@archlinux.org>
+ *  Copyright (c) 2008-2013 Pacman Development Team <pacman-dev@archlinux.org>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -17,8 +17,6 @@
  *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-#include "config.h"
-
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
@@ -39,7 +37,7 @@
 
 #if HAVE_LIBGPGME
 #define CHECK_ERR(void) do { \
-		if(gpg_err_code(err) != GPG_ERR_NO_ERROR) { goto error; } \
+		if(gpg_err_code(err) != GPG_ERR_NO_ERROR) { goto gpg_error; } \
 	} while(0)
 
 /**
@@ -107,7 +105,7 @@ static alpm_list_t *list_sigsum(gpgme_sigsum_t sigsum)
 	sigsum_test_bit(sigsum, &summary, GPGME_SIGSUM_CRL_TOO_OLD, "crl too old");
 	/* A policy was not met.  */
 	sigsum_test_bit(sigsum, &summary, GPGME_SIGSUM_BAD_POLICY, "bad policy");
-	/* A system error occured.  */
+	/* A system error occurred.  */
 	sigsum_test_bit(sigsum, &summary, GPGME_SIGSUM_SYS_ERROR, "sys error");
 	/* Fallback case */
 	if(!sigsum) {
@@ -176,7 +174,7 @@ static int init_gpgme(alpm_handle_t *handle)
 	init = 1;
 	return 0;
 
-error:
+gpg_error:
 	_alpm_log(handle, ALPM_LOG_ERROR, _("GPGME error: %s\n"), gpgme_strerror(err));
 	RET_ERR(handle, ALPM_ERR_GPGME, -1);
 }
@@ -187,13 +185,18 @@ error:
  * @param fpr the fingerprint key ID to look up
  * @return 1 if key is known, 0 if key is unknown, -1 on error
  */
-static int key_in_keychain(alpm_handle_t *handle, const char *fpr)
+int _alpm_key_in_keychain(alpm_handle_t *handle, const char *fpr)
 {
 	gpgme_error_t err;
 	gpgme_ctx_t ctx;
 	gpgme_key_t key;
 	int ret = -1;
 
+	if(init_gpgme(handle)) {
+		/* pm_errno was set in gpgme_init() */
+		goto error;
+	}
+
 	memset(&ctx, 0, sizeof(ctx));
 	err = gpgme_new(&ctx);
 	CHECK_ERR();
@@ -210,10 +213,12 @@ static int key_in_keychain(alpm_handle_t *handle, const char *fpr)
 	} else {
 		_alpm_log(handle, ALPM_LOG_DEBUG, "gpg error: %s\n", gpgme_strerror(err));
 	}
+	gpgme_key_unref(key);
+
+gpg_error:
+	gpgme_release(ctx);
 
 error:
-	gpgme_key_unref(key);
-	gpgme_release(ctx);
 	return ret;
 }
 
@@ -276,9 +281,7 @@ static int key_search(alpm_handle_t *handle, const char *fpr,
 		}
 	}
 
-	if(gpg_err_code(err) != GPG_ERR_NO_ERROR) {
-		goto error;
-	}
+	CHECK_ERR();
 
 	/* should only get here if key actually exists */
 	pgpkey->data = key;
@@ -292,11 +295,34 @@ static int key_search(alpm_handle_t *handle, const char *fpr,
 	pgpkey->email = key->uids->email;
 	pgpkey->created = key->subkeys->timestamp;
 	pgpkey->expires = key->subkeys->expires;
-	gpgme_release(ctx);
-	return 1;
+	pgpkey->length = key->subkeys->length;
+	pgpkey->revoked = key->subkeys->revoked;
 
-error:
-	_alpm_log(handle, ALPM_LOG_DEBUG, "gpg error: %s\n", gpgme_strerror(err));
+	switch(key->subkeys->pubkey_algo) {
+		case GPGME_PK_RSA:
+		case GPGME_PK_RSA_E:
+		case GPGME_PK_RSA_S:
+			pgpkey->pubkey_algo = 'R';
+			break;
+
+		case GPGME_PK_DSA:
+			pgpkey->pubkey_algo = 'D';
+			break;
+
+		case GPGME_PK_ELG_E:
+		case GPGME_PK_ELG:
+		case GPGME_PK_ECDSA:
+		case GPGME_PK_ECDH:
+			pgpkey->pubkey_algo = 'E';
+			break;
+	}
+
+	ret = 1;
+
+gpg_error:
+	if(ret != 1) {
+		_alpm_log(handle, ALPM_LOG_DEBUG, "gpg error: %s\n", gpgme_strerror(err));
+	}
 	free(full_fpr);
 	gpgme_release(ctx);
 	return ret;
@@ -345,11 +371,52 @@ static int key_import(alpm_handle_t *handle, alpm_pgpkey_t *key)
 		ret = 0;
 	}
 
-error:
+gpg_error:
 	gpgme_release(ctx);
 	return ret;
 }
 
+/**
+ * Import a key defined by a fingerprint into the local keyring.
+ * @param handle the context handle
+ * @param fpr the fingerprint key ID to import
+ * @return 0 on success, -1 on error
+ */
+int _alpm_key_import(alpm_handle_t *handle, const char *fpr)
+{
+	int answer = 0, ret = -1;
+	alpm_pgpkey_t fetch_key;
+	memset(&fetch_key, 0, sizeof(fetch_key));
+
+	if(key_search(handle, fpr, &fetch_key) == 1) {
+		_alpm_log(handle, ALPM_LOG_DEBUG,
+				"unknown key, found %s on keyserver\n", fetch_key.uid);
+		if(!_alpm_access(handle, handle->gpgdir, "pubring.gpg", W_OK)) {
+			QUESTION(handle, ALPM_QUESTION_IMPORT_KEY,
+					&fetch_key, NULL, NULL, &answer);
+			if(answer) {
+				if(key_import(handle, &fetch_key) == 0) {
+					ret = 0;
+				} else {
+					_alpm_log(handle, ALPM_LOG_ERROR,
+							_("key \"%s\" could not be imported\n"), fetch_key.uid);
+				}
+			}
+		} else {
+			/* keyring directory was not writable, so we don't even try */
+			_alpm_log(handle, ALPM_LOG_WARNING,
+					_("key %s, \"%s\" found on keyserver, keyring is not writable\n"),
+					fetch_key.fingerprint, fetch_key.uid);
+		}
+	} else {
+		_alpm_log(handle, ALPM_LOG_ERROR,
+				_("key \"%s\" could not be looked up remotely\n"), fpr);
+	}
+	gpgme_key_unref(fetch_key.data);
+
+	return ret;
+}
+
 /**
  * Decode a loaded signature in base64 form.
  * @param base64_data the signature to attempt to decode
@@ -357,8 +424,10 @@ error:
  * @param data_len the length of the returned data
  * @return 0 on success, -1 on failure to properly decode
  */
-static int decode_signature(const char *base64_data,
-		unsigned char **data, size_t *data_len) {
+
+int _alpm_decode_signature(const char *base64_data,
+		unsigned char **data, size_t *data_len)
+{
 	size_t len = strlen(base64_data);
 	unsigned char *usline = (unsigned char *)base64_data;
 	/* reasonable allocation of expected length is 3/4 of encoded length */
@@ -418,8 +487,13 @@ int _alpm_gpgme_checksig(alpm_handle_t *handle, const char *path,
 
 	if(!base64_sig) {
 		sigpath = _alpm_sigpath(handle, path);
-		/* this will just help debugging */
-		_alpm_access(handle, NULL, sigpath, R_OK);
+		if(_alpm_access(handle, NULL, sigpath, R_OK) != 0
+				|| (sigfile = fopen(sigpath, "rb")) == NULL) {
+			_alpm_log(handle, ALPM_LOG_DEBUG, "sig path %s could not be opened\n",
+					sigpath);
+			handle->pm_errno = ALPM_ERR_SIG_MISSING;
+			goto error;
+		}
 	}
 
 	/* does the file we are verifying exist? */
@@ -429,17 +503,6 @@ int _alpm_gpgme_checksig(alpm_handle_t *handle, const char *path,
 		goto error;
 	}
 
-	/* does the sig file exist (if we didn't get the data directly)? */
-	if(!base64_sig) {
-		sigfile = fopen(sigpath, "rb");
-		if(sigfile == NULL) {
-			_alpm_log(handle, ALPM_LOG_DEBUG, "sig path %s could not be opened\n",
-					sigpath);
-			handle->pm_errno = ALPM_ERR_SIG_MISSING;
-			goto error;
-		}
-	}
-
 	if(init_gpgme(handle)) {
 		/* pm_errno was set in gpgme_init() */
 		goto error;
@@ -462,7 +525,7 @@ int _alpm_gpgme_checksig(alpm_handle_t *handle, const char *path,
 	if(base64_sig) {
 		/* memory-based, we loaded it from a sync DB */
 		size_t data_len;
-		int decode_ret = decode_signature(base64_sig,
+		int decode_ret = _alpm_decode_signature(base64_sig,
 				&decoded_sigdata, &data_len);
 		if(decode_ret) {
 			handle->pm_errno = ALPM_ERR_SIG_INVALID;
@@ -510,6 +573,12 @@ int _alpm_gpgme_checksig(alpm_handle_t *handle, const char *path,
 		alpm_list_free(summary_list);
 		_alpm_log(handle, ALPM_LOG_DEBUG, "status: %s\n", gpgme_strerror(gpgsig->status));
 		_alpm_log(handle, ALPM_LOG_DEBUG, "timestamp: %lu\n", gpgsig->timestamp);
+
+		if((time_t)gpgsig->timestamp > time(NULL)) {
+			_alpm_log(handle, ALPM_LOG_DEBUG,
+					"signature timestamp is greater than system time.\n");
+		}
+
 		_alpm_log(handle, ALPM_LOG_DEBUG, "exp_timestamp: %lu\n", gpgsig->exp_timestamp);
 		_alpm_log(handle, ALPM_LOG_DEBUG, "validity: %s; reason: %s\n",
 				string_validity(gpgsig->validity),
@@ -611,10 +680,16 @@ error:
 }
 
 #else  /* HAVE_LIBGPGME */
-static int key_in_keychain(alpm_handle_t UNUSED *handle, const char UNUSED *fpr)
+int _alpm_key_in_keychain(alpm_handle_t UNUSED *handle, const char UNUSED *fpr)
 {
 	return -1;
 }
+
+int _alpm_key_import(alpm_handle_t UNUSED *handle, const char UNUSED *fpr)
+{
+	return -1;
+}
+
 int _alpm_gpgme_checksig(alpm_handle_t UNUSED *handle, const char UNUSED *path,
 		const char UNUSED *base64_sig, alpm_siglist_t UNUSED *siglist)
 {
@@ -790,44 +865,16 @@ int _alpm_process_siglist(alpm_handle_t *handle, const char *identifier,
 			case ALPM_SIGSTATUS_KEY_UNKNOWN:
 				/* ensure this key is still actually unknown; we may have imported it
 				 * on an earlier call to this function. */
-				if(key_in_keychain(handle, result->key.fingerprint) == 1) {
+				if(_alpm_key_in_keychain(handle, result->key.fingerprint) == 1) {
 					break;
 				}
 				_alpm_log(handle, ALPM_LOG_ERROR,
 						_("%s: key \"%s\" is unknown\n"), identifier, name);
-#ifdef HAVE_LIBGPGME
-				{
-					int answer;
-					alpm_pgpkey_t fetch_key;
-					memset(&fetch_key, 0, sizeof(fetch_key));
 
-					if(key_search(handle, result->key.fingerprint, &fetch_key) == 1) {
-						_alpm_log(handle, ALPM_LOG_DEBUG,
-								"unknown key, found %s on keyserver\n", fetch_key.uid);
-						if(!_alpm_access(handle, handle->gpgdir, "pubring.gpg", W_OK)) {
-							QUESTION(handle, ALPM_QUESTION_IMPORT_KEY,
-									&fetch_key, NULL, NULL, &answer);
-							if(answer) {
-								if(key_import(handle, &fetch_key) == 0) {
-									retry = 1;
-								} else {
-									_alpm_log(handle, ALPM_LOG_ERROR,
-											_("key \"%s\" could not be imported\n"), fetch_key.uid);
-								}
-							}
-						} else {
-							/* keyring directory was not writable, so we don't even try */
-							_alpm_log(handle, ALPM_LOG_WARNING,
-									_("key %s, \"%s\" found on keyserver, keyring is not writable\n"),
-									fetch_key.fingerprint, fetch_key.uid);
-						}
-					} else {
-						_alpm_log(handle, ALPM_LOG_ERROR,
-								_("key \"%s\" could not be looked up remotely\n"), name);
-					}
-					gpgme_key_unref(fetch_key.data);
+				if(_alpm_key_import(handle, result->key.fingerprint) == 0) {
+					retry = 1;
 				}
-#endif
+
 				break;
 			case ALPM_SIGSTATUS_KEY_DISABLED:
 				_alpm_log(handle, ALPM_LOG_ERROR,
@@ -910,4 +957,117 @@ int SYMEXPORT alpm_siglist_cleanup(alpm_siglist_t *siglist)
 	return 0;
 }
 
+/**
+ * Extract the Issuer Key ID from a signature
+ * @param sig PGP signature
+ * @param len length of signature
+ * @param keys a pointer to storage for key IDs
+ * @return 0 on success, -1 on error
+ */
+int _alpm_extract_keyid(alpm_handle_t *handle, const char *identifier,
+		const unsigned char *sig, const size_t len, alpm_list_t **keys)
+{
+	size_t pos, spos, blen, hlen, ulen, slen;
+	pos = 0;
+
+	while(pos < len) {
+		if(!(sig[pos] & 0x80)) {
+			_alpm_log(handle, ALPM_LOG_ERROR,
+					_("%s: signature format error"), identifier);
+			return -1;
+		}
+
+		if(sig[pos] & 0x40) {
+			/* "new" packet format is not supported */
+			_alpm_log(handle, ALPM_LOG_ERROR,
+					_("%s: unsupported signature format"), identifier);
+			return -1;
+		}
+
+		if(((sig[pos] & 0x3f) >> 2) != 2) {
+			/* signature is not a "Signature Packet" */
+			_alpm_log(handle, ALPM_LOG_ERROR,
+					_("%s: signature format error"), identifier);
+			return -1;
+		}
+
+		switch(sig[pos] & 0x03) {
+			case 0:
+				blen = sig[pos + 1];
+				pos = pos + 2;
+				break;
+
+			case 1:
+				blen = (sig[pos + 1] << 8) | sig[pos + 2];
+				pos = pos + 3;
+				break;
+
+			case 2:
+				blen = (sig[pos + 1] << 24) | (sig[pos + 2] << 16) | (sig[pos + 3] << 8) | sig[pos + 4];
+				pos = pos + 5;
+				break;
+
+			case 3:
+				/* partial body length not supported */
+				_alpm_log(handle, ALPM_LOG_ERROR,
+					_("%s: unsupported signature format"), identifier);
+				return -1;
+		}
+
+		if(sig[pos] != 4) {
+			/* only support version 4 signature packet format */
+			_alpm_log(handle, ALPM_LOG_ERROR,
+					_("%s: unsupported signature format"), identifier);
+			return -1;
+		}
+
+		if(sig[pos + 1] != 0x00) {
+			/* not a signature of a binary document */
+			_alpm_log(handle, ALPM_LOG_ERROR,
+					_("%s: signature format error"), identifier);
+			return -1;
+		}
+
+		pos = pos + 4;
+
+		hlen = (sig[pos] << 8) | sig[pos + 1];
+		pos = pos + hlen + 2;
+
+		ulen = (sig[pos] << 8) | sig[pos + 1];
+		pos = pos + 2;
+
+		spos = pos;
+
+		while(spos < pos + ulen) {
+			if(sig[spos] < 192) {
+				slen = sig[spos];
+				spos = spos + 1;
+			} else if(sig[spos] < 255) {
+				slen = (sig[spos] << 8) | sig[spos + 1];
+				spos = spos + 2;
+			} else {
+				slen = (sig[spos + 1] << 24) | (sig[spos + 2] << 16) | (sig[spos + 3] << 8) | sig[spos + 4];
+				spos = spos + 5;
+			}
+
+			if(sig[spos] == 16) {
+				/* issuer key ID */
+				char key[17];
+				size_t i;
+				for (i = 0; i < 8; i++) {
+					sprintf(&key[i * 2], "%02X", sig[spos + i + 1]);
+				}
+				*keys = alpm_list_add(*keys, strdup(key));
+				break;
+			}
+
+			spos = spos + slen;
+		}
+
+		pos = pos + (blen - hlen - 8);
+	}
+
+	return 0;
+}
+
 /* vim: set ts=2 sw=2 noet: */
