@@ -1,7 +1,7 @@
 /*
  *  signing.h
  *
- *  Copyright (c) 2008-2011 Pacman Development Team <pacman-dev@archlinux.org>
+ *  Copyright (c) 2008-2013 Pacman Development Team <pacman-dev@archlinux.org>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -31,6 +31,14 @@ int _alpm_check_pgp_helper(alpm_handle_t *handle, const char *path,
 int _alpm_process_siglist(alpm_handle_t *handle, const char *identifier,
 		alpm_siglist_t *siglist, int optional, int marginal, int unknown);
 
+int _alpm_key_in_keychain(alpm_handle_t *handle, const char *fpr);
+int _alpm_key_import(alpm_handle_t *handle, const char *fpr);
+
+int _alpm_decode_signature(const char *base64_data,
+		unsigned char **data, size_t *data_len);
+int _alpm_extract_keyid(alpm_handle_t *handle, const char *identifier,
+		const unsigned char *sig, const size_t len, alpm_list_t **keys);
+
 #endif /* _ALPM_SIGNING_H */
 
 /* vim: set ts=2 sw=2 noet: */
