@@ -1,7 +1,7 @@
 /*
  *  sync.c
  *
- *  Copyright (c) 2006-2011 Pacman Development Team <pacman-dev@archlinux.org>
+ *  Copyright (c) 2006-2013 Pacman Development Team <pacman-dev@archlinux.org>
  *  Copyright (c) 2002-2006 by Judd Vinet <jvinet@zeroflux.org>
  *  Copyright (c) 2005 by Aurelien Foret <orelien@chez.com>
  *  Copyright (c) 2005 by Christian Hamar <krics@linuxforum.hu>
@@ -21,8 +21,6 @@
  *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-#include "config.h"
-
 #include <sys/types.h> /* off_t */
 #include <stdlib.h>
 #include <stdio.h>
@@ -127,7 +125,8 @@ static alpm_list_t *check_replacers(alpm_handle_t *handle, alpm_pkg_t *lpkg,
 	alpm_list_t *replacers = NULL;
 	alpm_list_t *k;
 	_alpm_log(handle, ALPM_LOG_DEBUG,
-			"searching for replacements for %s\n", lpkg->name);
+			"searching for replacements for %s in %s\n",
+			lpkg->name, sdb->treename);
 	for(k = _alpm_db_get_pkgcache(sdb); k; k = k->next) {
 		int found = 0;
 		alpm_pkg_t *spkg = k->data;
@@ -160,7 +159,7 @@ static alpm_list_t *check_replacers(alpm_handle_t *handle, alpm_pkg_t *lpkg,
 
 			/* If spkg is already in the target list, we append lpkg to spkg's
 			 * removes list */
-			tpkg = _alpm_pkg_find(handle->trans->add, spkg->name);
+			tpkg = alpm_pkg_find(handle->trans->add, spkg->name);
 			if(tpkg) {
 				/* sanity check, multiple repos can contain spkg->name */
 				if(tpkg->origin_data.db != sdb) {
@@ -205,29 +204,28 @@ int SYMEXPORT alpm_sync_sysupgrade(alpm_handle_t *handle, int enable_downgrade)
 	for(i = _alpm_db_get_pkgcache(handle->db_local); i; i = i->next) {
 		alpm_pkg_t *lpkg = i->data;
 
-		if(_alpm_pkg_find(trans->add, lpkg->name)) {
+		if(alpm_pkg_find(trans->add, lpkg->name)) {
 			_alpm_log(handle, ALPM_LOG_DEBUG, "%s is already in the target list -- skipping\n", lpkg->name);
 			continue;
 		}
 
-		/* Search for literal then replacers in each sync database. */
+		/* Search for replacers then literal (if no replacer) in each sync database. */
 		for(j = handle->dbs_sync; j; j = j->next) {
 			alpm_db_t *sdb = j->data;
+			alpm_list_t *replacers = check_replacers(handle, lpkg, sdb);
 			/* Check sdb */
-			alpm_pkg_t *spkg = _alpm_db_get_pkgfromcache(sdb, lpkg->name);
-			int literal_upgrade = 0;
-			if(spkg) {
-				literal_upgrade = check_literal(handle, lpkg, spkg, enable_downgrade);
-				if(literal_upgrade) {
-					trans->add = alpm_list_add(trans->add, spkg);
-				}
+			if(replacers) {
+				trans->add = alpm_list_join(trans->add, replacers);
 				/* jump to next local package */
 				break;
 			} else {
-				alpm_list_t *replacers;
-				replacers = check_replacers(handle, lpkg, sdb);
-				if(replacers) {
-					trans->add = alpm_list_join(trans->add, replacers);
+				alpm_pkg_t *spkg = _alpm_db_get_pkgfromcache(sdb, lpkg->name);
+				if(spkg) {
+					if(check_literal(handle, lpkg, spkg, enable_downgrade)) {
+						trans->add = alpm_list_add(trans->add, spkg);
+					}
+					/* jump to next local package */
+					break;
 				}
 			}
 		}
@@ -250,7 +248,7 @@ alpm_list_t SYMEXPORT *alpm_find_group_pkgs(alpm_list_t *dbs,
 
 	for(i = dbs; i; i = i->next) {
 		alpm_db_t *db = i->data;
-		alpm_group_t *grp = alpm_db_readgroup(db, name);
+		alpm_group_t *grp = alpm_db_get_group(db, name);
 
 		if(!grp)
 			continue;
@@ -258,7 +256,7 @@ alpm_list_t SYMEXPORT *alpm_find_group_pkgs(alpm_list_t *dbs,
 		for(j = grp->packages; j; j = j->next) {
 			alpm_pkg_t *pkg = j->data;
 
-			if(_alpm_pkg_find(ignorelist, pkg->name)) {
+			if(alpm_pkg_find(ignorelist, pkg->name)) {
 				continue;
 			}
 			if(_alpm_pkg_should_ignore(db->handle, pkg)) {
@@ -269,7 +267,7 @@ alpm_list_t SYMEXPORT *alpm_find_group_pkgs(alpm_list_t *dbs,
 				if(!install)
 					continue;
 			}
-			if(!_alpm_pkg_find(pkgs, pkg->name)) {
+			if(!alpm_pkg_find(pkgs, pkg->name)) {
 				pkgs = alpm_list_add(pkgs, pkg);
 			}
 		}
@@ -290,7 +288,7 @@ static int compute_download_size(alpm_pkg_t *newpkg)
 	alpm_handle_t *handle = newpkg->handle;
 	int ret = 0;
 
-	if(newpkg->origin != PKG_FROM_SYNCDB) {
+	if(newpkg->origin != ALPM_PKG_FROM_SYNCDB) {
 		newpkg->infolevel |= INFRQ_DSIZE;
 		newpkg->download_size = 0;
 		return 0;
@@ -320,13 +318,13 @@ static int compute_download_size(alpm_pkg_t *newpkg)
 
 		/* tell the caller that we have a partial */
 		ret = 1;
-	} else if(handle->usedelta) {
+	} else if(handle->deltaratio > 0.0) {
 		off_t dltsize;
 
 		dltsize = _alpm_shortest_delta_path(handle, newpkg->deltas,
 				newpkg->filename, &newpkg->delta_path);
 
-		if(newpkg->delta_path && (dltsize < newpkg->size * MAX_DELTA_RATIO)) {
+		if(newpkg->delta_path && (dltsize < newpkg->size * handle->deltaratio)) {
 			_alpm_log(handle, ALPM_LOG_DEBUG, "using delta size\n");
 			size = dltsize;
 		} else {
@@ -367,7 +365,7 @@ int _alpm_sync_prepare(alpm_handle_t *handle, alpm_list_t **data)
 
 	for(i = trans->add; i; i = i->next) {
 		alpm_pkg_t *spkg = i->data;
-		from_sync += (spkg->origin == PKG_FROM_SYNCDB);
+		from_sync += (spkg->origin == ALPM_PKG_FROM_SYNCDB);
 	}
 
 	/* ensure all sync database are valid if we will be using them */
@@ -397,6 +395,8 @@ int _alpm_sync_prepare(alpm_handle_t *handle, alpm_list_t **data)
 			alpm_pkg_t *spkg = i->data;
 			for(j = spkg->removes; j; j = j->next) {
 				remove = alpm_list_add(remove, j->data);
+				/* do not re-install a (replaced) package even if an update is available */
+				trans->add = alpm_list_remove(trans->add, j->data, _alpm_pkg_cmp, NULL);
 			}
 		}
 
@@ -423,7 +423,7 @@ int _alpm_sync_prepare(alpm_handle_t *handle, alpm_list_t **data)
 		   see if they'd like to ignore them rather than failing the sync */
 		if(unresolvable != NULL) {
 			int remove_unresolvable = 0;
-			enum _alpm_errno_t saved_err = handle->pm_errno;
+			alpm_errno_t saved_err = handle->pm_errno;
 			QUESTION(handle, ALPM_QUESTION_REMOVE_PKGS, unresolvable,
 					NULL, NULL, &remove_unresolvable);
 			if(remove_unresolvable) {
@@ -449,7 +449,7 @@ int _alpm_sync_prepare(alpm_handle_t *handle, alpm_list_t **data)
 		/* Set DEPEND reason for pulled packages */
 		for(i = resolved; i; i = i->next) {
 			alpm_pkg_t *pkg = i->data;
-			if(!_alpm_pkg_find(trans->add, pkg->name)) {
+			if(!alpm_pkg_find(trans->add, pkg->name)) {
 				pkg->reason = ALPM_PKG_REASON_DEPEND;
 			}
 		}
@@ -483,8 +483,8 @@ int _alpm_sync_prepare(alpm_handle_t *handle, alpm_list_t **data)
 			alpm_pkg_t *rsync, *sync, *sync1, *sync2;
 
 			/* have we already removed one of the conflicting targets? */
-			sync1 = _alpm_pkg_find(trans->add, conflict->package1);
-			sync2 = _alpm_pkg_find(trans->add, conflict->package2);
+			sync1 = alpm_pkg_find(trans->add, conflict->package1);
+			sync2 = alpm_pkg_find(trans->add, conflict->package2);
 			if(!sync1 || !sync2) {
 				continue;
 			}
@@ -540,13 +540,14 @@ int _alpm_sync_prepare(alpm_handle_t *handle, alpm_list_t **data)
 
 		for(i = deps; i; i = i->next) {
 			alpm_conflict_t *conflict = i->data;
+			int doremove = 0;
+			int found = 0;
 
 			/* if conflict->package2 (the local package) is not elected for removal,
 			   we ask the user */
-			int found = 0;
 			for(j = trans->add; j && !found; j = j->next) {
 				alpm_pkg_t *spkg = j->data;
-				if(_alpm_pkg_find(spkg->removes, conflict->package2)) {
+				if(alpm_pkg_find(spkg->removes, conflict->package2)) {
 					found = 1;
 				}
 			}
@@ -557,13 +558,12 @@ int _alpm_sync_prepare(alpm_handle_t *handle, alpm_list_t **data)
 			_alpm_log(handle, ALPM_LOG_DEBUG, "package '%s' conflicts with '%s'\n",
 					conflict->package1, conflict->package2);
 
-			alpm_pkg_t *sync = _alpm_pkg_find(trans->add, conflict->package1);
-			alpm_pkg_t *local = _alpm_db_get_pkgfromcache(handle->db_local, conflict->package2);
-			int doremove = 0;
 			QUESTION(handle, ALPM_QUESTION_CONFLICT_PKG, conflict->package1,
 							conflict->package2, conflict->reason->name, &doremove);
 			if(doremove) {
 				/* append to the removes list */
+				alpm_pkg_t *sync = alpm_pkg_find(trans->add, conflict->package1);
+				alpm_pkg_t *local = _alpm_db_get_pkgfromcache(handle->db_local, conflict->package2);
 				_alpm_log(handle, ALPM_LOG_DEBUG, "electing '%s' for removal\n", conflict->package2);
 				sync->removes = alpm_list_add(sync->removes, local);
 			} else { /* abort */
@@ -591,7 +591,7 @@ int _alpm_sync_prepare(alpm_handle_t *handle, alpm_list_t **data)
 		alpm_pkg_t *spkg = i->data;
 		for(j = spkg->removes; j; j = j->next) {
 			alpm_pkg_t *rpkg = j->data;
-			if(!_alpm_pkg_find(trans->remove, rpkg->name)) {
+			if(!alpm_pkg_find(trans->remove, rpkg->name)) {
 				alpm_pkg_t *copy;
 				_alpm_log(handle, ALPM_LOG_DEBUG, "adding '%s' to remove list\n", rpkg->name);
 				if(_alpm_pkg_dup(rpkg, &copy) == -1) {
@@ -695,11 +695,11 @@ static int apply_deltas(alpm_handle_t *handle)
 			} else {
 				/* len = cachedir len + from len + '/' + null */
 				len = strlen(cachedir) + strlen(d->from) + 2;
-				CALLOC(from, len, sizeof(char), RET_ERR(handle, ALPM_ERR_MEMORY, 1));
+				MALLOC(from, len, RET_ERR(handle, ALPM_ERR_MEMORY, 1));
 				snprintf(from, len, "%s/%s", cachedir, d->from);
 			}
 			len = strlen(cachedir) + strlen(d->to) + 2;
-			CALLOC(to, len, sizeof(char), RET_ERR(handle, ALPM_ERR_MEMORY, 1));
+			MALLOC(to, len, free(from); RET_ERR(handle, ALPM_ERR_MEMORY, 1));
 			snprintf(to, len, "%s/%s", cachedir, d->to);
 
 			/* build the patch command */
@@ -757,7 +757,7 @@ static int apply_deltas(alpm_handle_t *handle)
  * @return 1 if file was removed, 0 otherwise
  */
 static int prompt_to_delete(alpm_handle_t *handle, const char *filepath,
-		enum _alpm_errno_t reason)
+		alpm_errno_t reason)
 {
 	int doremove = 0;
 	QUESTION(handle, ALPM_QUESTION_CORRUPTED_PKG, (char *)filepath,
@@ -782,7 +782,7 @@ static int validate_deltas(alpm_handle_t *handle, alpm_list_t *deltas)
 		alpm_delta_t *d = i->data;
 		char *filepath = _alpm_filecache_find(handle, d->delta);
 
-		if(_alpm_test_checksum(filepath, d->delta_md5, ALPM_CSUM_MD5)) {
+		if(_alpm_test_checksum(filepath, d->delta_md5, ALPM_PKG_VALIDATION_MD5SUM)) {
 			errors = alpm_list_add(errors, filepath);
 		} else {
 			FREE(filepath);
@@ -803,11 +803,96 @@ static int validate_deltas(alpm_handle_t *handle, alpm_list_t *deltas)
 	return 0;
 }
 
+static struct dload_payload *build_payload(alpm_handle_t *handle,
+		const char *filename, size_t size, alpm_list_t *servers)
+{
+		struct dload_payload *payload;
+
+		CALLOC(payload, 1, sizeof(*payload), RET_ERR(handle, ALPM_ERR_MEMORY, NULL));
+		STRDUP(payload->remote_name, filename, RET_ERR(handle, ALPM_ERR_MEMORY, NULL));
+		payload->max_size = size;
+		payload->servers = servers;
+		return payload;
+}
+
+static int find_dl_candidates(alpm_db_t *repo, alpm_list_t **files, alpm_list_t **deltas)
+{
+	alpm_list_t *i;
+	alpm_handle_t *handle = repo->handle;
+
+	for(i = handle->trans->add; i; i = i->next) {
+		alpm_pkg_t *spkg = i->data;
+
+		if(spkg->origin != ALPM_PKG_FROM_FILE && repo == spkg->origin_data.db) {
+			alpm_list_t *delta_path = spkg->delta_path;
+
+			if(!repo->servers) {
+				handle->pm_errno = ALPM_ERR_SERVER_NONE;
+				_alpm_log(handle, ALPM_LOG_ERROR, "%s: %s\n",
+						alpm_strerror(handle->pm_errno), repo->treename);
+				return 1;
+			}
+
+			if(delta_path) {
+				/* using deltas */
+				alpm_list_t *dlts;
+				for(dlts = delta_path; dlts; dlts = dlts->next) {
+					alpm_delta_t *delta = dlts->data;
+					if(delta->download_size != 0) {
+						struct dload_payload *payload = build_payload(
+								handle, delta->delta, delta->delta_size, repo->servers);
+						ASSERT(payload, return -1);
+						*files = alpm_list_add(*files, payload);
+					}
+					/* keep a list of all the delta files for md5sums */
+					*deltas = alpm_list_add(*deltas, delta);
+				}
+
+			} else if(spkg->download_size != 0) {
+				struct dload_payload *payload;
+				ASSERT(spkg->filename != NULL, RET_ERR(handle, ALPM_ERR_PKG_INVALID_NAME, -1));
+				payload = build_payload(handle, spkg->filename, spkg->size, repo->servers);
+				ASSERT(payload, return -1);
+				*files = alpm_list_add(*files, payload);
+			}
+		}
+	}
+
+	return 0;
+}
+
+static int download_single_file(alpm_handle_t *handle, struct dload_payload *payload,
+		const char *cachedir)
+{
+	const alpm_list_t *server;
+
+	payload->handle = handle;
+	payload->allow_resume = 1;
+
+	for(server = payload->servers; server; server = server->next) {
+		const char *server_url = server->data;
+		size_t len;
+
+		/* print server + filename into a buffer */
+		len = strlen(server_url) + strlen(payload->remote_name) + 2;
+		MALLOC(payload->fileurl, len, RET_ERR(handle, ALPM_ERR_MEMORY, -1));
+		snprintf(payload->fileurl, len, "%s/%s", server_url, payload->remote_name);
+
+		if(_alpm_download(payload, cachedir, NULL, NULL) != -1) {
+			return 0;
+		}
+
+		FREE(payload->fileurl);
+		payload->unlink_on_fail = 0;
+	}
+
+	return -1;
+}
+
 static int download_files(alpm_handle_t *handle, alpm_list_t **deltas)
 {
 	const char *cachedir;
-	alpm_list_t *i, *j;
-	alpm_list_t *files = NULL;
+	alpm_list_t *i, *files = NULL;
 	int errors = 0;
 
 	cachedir = _alpm_filecache_setup(handle);
@@ -826,93 +911,53 @@ static int download_files(alpm_handle_t *handle, alpm_list_t **deltas)
 		handle->totaldlcb(total_size);
 	}
 
-	/* group sync records by repository and download */
 	for(i = handle->dbs_sync; i; i = i->next) {
-		alpm_db_t *current = i->data;
+		errors += find_dl_candidates(i->data, &files, deltas);
+	}
 
-		for(j = handle->trans->add; j; j = j->next) {
-			alpm_pkg_t *spkg = j->data;
+	if(files) {
+		/* check for necessary disk space for download */
+		if(handle->checkspace) {
+			off_t *file_sizes;
+			size_t idx, num_files;
+			int ret;
 
-			if(spkg->origin != PKG_FROM_FILE && current == spkg->origin_data.db) {
-				alpm_list_t *delta_path = spkg->delta_path;
-				if(delta_path) {
-					/* using deltas */
-					alpm_list_t *dlts;
-					for(dlts = delta_path; dlts; dlts = dlts->next) {
-						alpm_delta_t *delta = dlts->data;
-						if(delta->download_size != 0) {
-							struct dload_payload *dpayload;
+			_alpm_log(handle, ALPM_LOG_DEBUG, "checking available disk space for download\n");
 
-							CALLOC(dpayload, 1, sizeof(*dpayload), RET_ERR(handle, ALPM_ERR_MEMORY, -1));
-							STRDUP(dpayload->remote_name, delta->delta, RET_ERR(handle, ALPM_ERR_MEMORY, -1));
-							dpayload->max_size = delta->delta_size;
+			num_files = alpm_list_count(files);
+			CALLOC(file_sizes, num_files, sizeof(off_t), goto finish);
 
-							files = alpm_list_add(files, dpayload);
-						}
-						/* keep a list of all the delta files for md5sums */
-						*deltas = alpm_list_add(*deltas, delta);
-					}
+			for(i = files, idx = 0; i; i = i->next, idx++) {
+				const struct dload_payload *payload = i->data;
+				file_sizes[idx] = payload->max_size;
+			}
 
-				} else if(spkg->download_size != 0) {
-					struct dload_payload *payload;
-
-					ASSERT(spkg->filename != NULL, RET_ERR(handle, ALPM_ERR_PKG_INVALID_NAME, -1));
-					CALLOC(payload, 1, sizeof(*payload), RET_ERR(handle, ALPM_ERR_MEMORY, -1));
-					STRDUP(payload->remote_name, spkg->filename, RET_ERR(handle, ALPM_ERR_MEMORY, -1));
-					payload->max_size = spkg->size;
-
-					files = alpm_list_add(files, payload);
-				}
+			ret = _alpm_check_downloadspace(handle, cachedir, num_files, file_sizes);
+			free(file_sizes);
 
+			if(ret != 0) {
+				errors++;
+				goto finish;
 			}
 		}
 
-		if(files) {
-			if(!current->servers) {
-				handle->pm_errno = ALPM_ERR_SERVER_NONE;
-				_alpm_log(handle, ALPM_LOG_ERROR, "%s: %s\n",
-						alpm_strerror(handle->pm_errno), current->treename);
+		EVENT(handle, ALPM_EVENT_RETRIEVE_START, NULL, NULL);
+		for(i = files; i; i = i->next) {
+			if(download_single_file(handle, i->data, cachedir) == -1) {
 				errors++;
-				continue;
+				_alpm_log(handle, ALPM_LOG_WARNING, _("failed to retrieve some files\n"));
 			}
-
-			EVENT(handle, ALPM_EVENT_RETRIEVE_START, current->treename, NULL);
-			for(j = files; j; j = j->next) {
-				struct dload_payload *payload = j->data;
-				alpm_list_t *server;
-				int ret = -1;
-				for(server = current->servers; server; server = server->next) {
-					const char *server_url = server->data;
-					size_t len;
-
-					/* print server + filename into a buffer */
-					len = strlen(server_url) + strlen(payload->remote_name) + 2;
-					MALLOC(payload->fileurl, len, RET_ERR(handle, ALPM_ERR_MEMORY, -1));
-					snprintf(payload->fileurl, len, "%s/%s", server_url, payload->remote_name);
-					payload->handle = handle;
-					payload->allow_resume = 1;
-
-					ret = _alpm_download(payload, cachedir, NULL);
-					if(ret != -1) {
-						break;
-					}
-					FREE(payload->fileurl);
-					payload->unlink_on_fail = 0;
-				}
-				if(ret == -1) {
-					errors++;
-					_alpm_log(handle, ALPM_LOG_WARNING, _("failed to retrieve some files from %s\n"),
-							current->treename);
-				}
-			}
-
-			alpm_list_free_inner(files, (alpm_list_fn_free)_alpm_dload_payload_reset);
-			FREELIST(files);
 		}
 	}
 
-	for(j = handle->trans->add; j; j = j->next) {
-		alpm_pkg_t *pkg = j->data;
+finish:
+	if(files) {
+		alpm_list_free_inner(files, (alpm_list_fn_free)_alpm_dload_payload_reset);
+		FREELIST(files);
+	}
+
+	for(i = handle->trans->add; i; i = i->next) {
+		alpm_pkg_t *pkg = i->data;
 		pkg->infolevel &= ~INFRQ_DSIZE;
 		pkg->download_size = 0;
 	}
@@ -925,6 +970,78 @@ static int download_files(alpm_handle_t *handle, alpm_list_t **deltas)
 	return errors;
 }
 
+#ifdef HAVE_LIBGPGME
+static int check_keyring(alpm_handle_t *handle)
+{
+	size_t current = 0, numtargs;
+	alpm_list_t *i, *errors = NULL;
+
+	EVENT(handle, ALPM_EVENT_KEYRING_START, NULL, NULL);
+
+	numtargs = alpm_list_count(handle->trans->add);
+
+	for(i = handle->trans->add; i; i = i->next, current++) {
+		alpm_pkg_t *pkg = i->data;
+		alpm_siglevel_t level;
+
+		int percent = (current * 100) / numtargs;
+		PROGRESS(handle, ALPM_PROGRESS_KEYRING_START, "", percent,
+				numtargs, current);
+
+		if(pkg->origin == ALPM_PKG_FROM_FILE) {
+			continue; /* pkg_load() has been already called, this package is valid */
+		}
+
+		level = alpm_db_get_siglevel(alpm_pkg_get_db(pkg));
+		if((level & ALPM_SIG_PACKAGE) && pkg->base64_sig) {
+			unsigned char *decoded_sigdata = NULL;
+			size_t data_len;
+			int decode_ret = _alpm_decode_signature(pkg->base64_sig,
+					&decoded_sigdata, &data_len);
+			if(decode_ret == 0) {
+				alpm_list_t *keys = NULL;
+				if(_alpm_extract_keyid(handle, pkg->name, decoded_sigdata,
+							data_len, &keys) == 0) {
+					alpm_list_t *k;
+					for(k = keys; k; k = k->next) {
+						char *key = k->data;
+						if(_alpm_key_in_keychain(handle, key) == 0) {
+							if(!alpm_list_find_str(errors, key)) {
+								errors = alpm_list_add(errors, strdup(key));
+							}
+						}
+					}
+					FREELIST(keys);
+				}
+			}
+		}
+	}
+
+	PROGRESS(handle, ALPM_PROGRESS_KEYRING_START, "", 100,
+			numtargs, current);
+	EVENT(handle, ALPM_EVENT_KEYRING_DONE, NULL, NULL);
+
+	if(errors) {
+		EVENT(handle, ALPM_EVENT_KEY_DOWNLOAD_START, NULL, NULL);
+		int fail = 0;
+		alpm_list_t *k;
+		for(k = errors; k; k = k->next) {
+			char *key = k->data;
+			if(_alpm_key_import(handle, key) == -1) {
+				fail = 1;
+			}
+		}
+		EVENT(handle, ALPM_EVENT_KEY_DOWNLOAD_DONE, NULL, NULL);
+		if(fail) {
+			_alpm_log(handle, ALPM_LOG_ERROR, _("required key missing from keyring\n"));
+			return -1;
+		}
+	}
+
+	return 0;
+}
+#endif /* HAVE_LIBGPGME */
+
 static int check_validity(alpm_handle_t *handle,
 		size_t total, size_t total_bytes)
 {
@@ -933,7 +1050,8 @@ static int check_validity(alpm_handle_t *handle,
 		char *path;
 		alpm_siglist_t *siglist;
 		alpm_siglevel_t level;
-		enum _alpm_errno_t error;
+		alpm_pkgvalidation_t validation;
+		alpm_errno_t error;
 	};
 	size_t current = 0, current_bytes = 0;
 	alpm_list_t *i, *errors = NULL;
@@ -942,12 +1060,12 @@ static int check_validity(alpm_handle_t *handle,
 	EVENT(handle, ALPM_EVENT_INTEGRITY_START, NULL, NULL);
 
 	for(i = handle->trans->add; i; i = i->next, current++) {
-		struct validity v = { i->data, NULL, NULL, 0, 0 };
+		struct validity v = { i->data, NULL, NULL, 0, 0, 0 };
 		int percent = (int)(((double)current_bytes / total_bytes) * 100);
 
 		PROGRESS(handle, ALPM_PROGRESS_INTEGRITY_START, "", percent,
 				total, current);
-		if(v.pkg->origin == PKG_FROM_FILE) {
+		if(v.pkg->origin == ALPM_PKG_FROM_FILE) {
 			continue; /* pkg_load() has been already called, this package is valid */
 		}
 
@@ -956,7 +1074,7 @@ static int check_validity(alpm_handle_t *handle,
 		v.level = alpm_db_get_siglevel(alpm_pkg_get_db(v.pkg));
 
 		if(_alpm_pkg_validate_internal(handle, v.path, v.pkg,
-					v.level, &v.siglist) == -1) {
+					v.level, &v.siglist, &v.validation) == -1) {
 			v.error = handle->pm_errno;
 			struct validity *invalid = malloc(sizeof(struct validity));
 			memcpy(invalid, &v, sizeof(struct validity));
@@ -965,6 +1083,7 @@ static int check_validity(alpm_handle_t *handle,
 			alpm_siglist_cleanup(v.siglist);
 			free(v.siglist);
 			free(v.path);
+			v.pkg->validation = v.validation;
 		}
 	}
 
@@ -973,15 +1092,14 @@ static int check_validity(alpm_handle_t *handle,
 	EVENT(handle, ALPM_EVENT_INTEGRITY_DONE, NULL, NULL);
 
 	if(errors) {
-		int tryagain = 0;
 		for(i = errors; i; i = i->next) {
 			struct validity *v = i->data;
 			if(v->error == ALPM_ERR_PKG_INVALID_SIG) {
-				int retry = _alpm_process_siglist(handle, v->pkg->name, v->siglist,
+				_alpm_process_siglist(handle, v->pkg->name, v->siglist,
 						v->level & ALPM_SIG_PACKAGE_OPTIONAL,
 						v->level & ALPM_SIG_PACKAGE_MARGINAL_OK,
 						v->level & ALPM_SIG_PACKAGE_UNKNOWN_OK);
-				tryagain += retry;
+				prompt_to_delete(handle, v->path, v->error);
 			} else if(v->error == ALPM_ERR_PKG_INVALID_CHECKSUM) {
 				prompt_to_delete(handle, v->path, v->error);
 			}
@@ -992,14 +1110,10 @@ static int check_validity(alpm_handle_t *handle,
 		}
 		alpm_list_free(errors);
 
-		if(tryagain == 0) {
-			if(!handle->pm_errno) {
-				RET_ERR(handle, ALPM_ERR_PKG_INVALID, -1);
-			}
-			return -1;
+		if(!handle->pm_errno) {
+			RET_ERR(handle, ALPM_ERR_PKG_INVALID, -1);
 		}
-		/* we were told at least once we can try again */
-		return 1;
+		return -1;
 	}
 
 	return 0;
@@ -1022,7 +1136,7 @@ static int load_packages(alpm_handle_t *handle, alpm_list_t **data,
 
 		PROGRESS(handle, ALPM_PROGRESS_LOAD_START, "", percent,
 				total, current);
-		if(spkg->origin == PKG_FROM_FILE) {
+		if(spkg->origin == ALPM_PKG_FROM_FILE) {
 			continue; /* pkg_load() has been already called, this package is valid */
 		}
 
@@ -1044,6 +1158,8 @@ static int load_packages(alpm_handle_t *handle, alpm_list_t **data,
 		free(filepath);
 		/* copy over the install reason */
 		pkgfile->reason = spkg->reason;
+		/* copy over validation method */
+		pkgfile->validation = spkg->validation;
 		i->data = pkgfile;
 		/* spkg has been removed from the target list, so we can free the
 		 * sync-specific fields */
@@ -1086,11 +1202,18 @@ int _alpm_sync_commit(alpm_handle_t *handle, alpm_list_t **data)
 		return -1;
 	}
 
+#if HAVE_LIBGPGME
+	/* make sure all required signatures are in keyring */
+	if(check_keyring(handle)) {
+		return -1;
+	}
+#endif
+
 	/* get the total size of all packages so we can adjust the progress bar more
 	 * realistically if there are small and huge packages involved */
 	for(i = trans->add; i; i = i->next) {
 		alpm_pkg_t *spkg = i->data;
-		if(spkg->origin != PKG_FROM_FILE) {
+		if(spkg->origin != ALPM_PKG_FROM_FILE) {
 			total_bytes += spkg->size;
 		}
 		total++;
@@ -1119,7 +1242,7 @@ int _alpm_sync_commit(alpm_handle_t *handle, alpm_list_t **data)
 	trans->state = STATE_COMMITING;
 
 	/* fileconflict check */
-	if(!(trans->flags & ALPM_TRANS_FLAG_FORCE)) {
+	if(!(trans->flags & ALPM_TRANS_FLAG_DBONLY)) {
 		EVENT(handle, ALPM_EVENT_FILECONFLICTS_START, NULL, NULL);
 
 		_alpm_log(handle, ALPM_LOG_DEBUG, "looking for file conflicts\n");
@@ -1144,7 +1267,7 @@ int _alpm_sync_commit(alpm_handle_t *handle, alpm_list_t **data)
 
 		_alpm_log(handle, ALPM_LOG_DEBUG, "checking available disk space\n");
 		if(_alpm_check_diskspace(handle) == -1) {
-			_alpm_log(handle, ALPM_LOG_ERROR, "%s\n", _("not enough free disk space"));
+			_alpm_log(handle, ALPM_LOG_ERROR, _("not enough free disk space\n"));
 			return -1;
 		}
 
