@@ -1,7 +1,7 @@
 /*
  *  query.c
  *
- *  Copyright (c) 2006-2011 Pacman Development Team <pacman-dev@archlinux.org>
+ *  Copyright (c) 2006-2013 Pacman Development Team <pacman-dev@archlinux.org>
  *  Copyright (c) 2002-2006 by Judd Vinet <jvinet@zeroflux.org>
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -18,13 +18,10 @@
  *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-#include "config.h"
-
 #include <stdlib.h>
 #include <stdio.h>
 #include <stdint.h>
 #include <limits.h>
-#include <string.h>
 #include <sys/stat.h>
 #include <unistd.h>
 #include <errno.h>
@@ -35,25 +32,11 @@
 /* pacman */
 #include "pacman.h"
 #include "package.h"
+#include "check.h"
 #include "conf.h"
 #include "util.h"
 
-static char *resolve_path(const char *file)
-{
-	char *str = NULL;
-
-	str = calloc(PATH_MAX, sizeof(char));
-	if(!str) {
-		return NULL;
-	}
-
-	if(!realpath(file, str)) {
-		free(str);
-		return NULL;
-	}
-
-	return str;
-}
+#define LOCAL_PREFIX "local/"
 
 /* check if filename exists in PATH */
 static int search_path(char **filename, struct stat *bufptr)
@@ -75,7 +58,7 @@ static int search_path(char **filename, struct stat *bufptr)
 
 		/* strip the trailing slash if one exists */
 		while(path[plen - 1] == '/') {
-				path[--plen] = '\0';
+			path[--plen] = '\0';
 		}
 
 		fullname = malloc(plen + flen + 2);
@@ -111,10 +94,10 @@ static int query_fileowner(alpm_list_t *targets)
 {
 	int ret = 0;
 	char path[PATH_MAX];
-	const char *root;
 	size_t rootlen;
 	alpm_list_t *t;
 	alpm_db_t *db_local;
+	alpm_list_t *packages;
 
 	/* This code is here for safety only */
 	if(targets == NULL) {
@@ -125,25 +108,47 @@ static int query_fileowner(alpm_list_t *targets)
 	/* Set up our root path buffer. We only need to copy the location of root in
 	 * once, then we can just overwrite whatever file was there on the previous
 	 * iteration. */
-	root = alpm_option_get_root(config->handle);
-	rootlen = strlen(root);
-	if(rootlen + 1 > PATH_MAX) {
-		/* we are in trouble here */
-		pm_printf(ALPM_LOG_ERROR, _("path too long: %s%s\n"), root, "");
+
+	/* resolve root now so any symlinks in it will only have to be resolved once */
+	if(!realpath(alpm_option_get_root(config->handle), path)) {
+		pm_printf(ALPM_LOG_ERROR, _("cannot determine real path for '%s': %s\n"),
+				path, strerror(errno));
 		return 1;
 	}
-	strcpy(path, root);
 
-	db_local = alpm_option_get_localdb(config->handle);
+	/* make sure there's enough room to append the package file to path */
+	rootlen = strlen(path);
+	if(rootlen + 2 > PATH_MAX) {
+		pm_printf(ALPM_LOG_ERROR, _("path too long: %s%s\n"), path, "");
+		return 1;
+	}
+
+	/* append trailing '/' removed by realpath */
+	if(path[rootlen - 1] != '/') {
+		path[rootlen++] = '/';
+		path[rootlen] = '\0';
+	}
+
+	db_local = alpm_get_localdb(config->handle);
+	packages = alpm_db_get_pkgcache(db_local);
 
 	for(t = targets; t; t = alpm_list_next(t)) {
-		char *filename, *dname, *rpath;
+		char *filename = NULL, *dname = NULL, *rpath = NULL;
 		const char *bname;
 		struct stat buf;
 		alpm_list_t *i;
-		int found = 0;
+		size_t len, is_dir, bname_len, pbname_len;
+		unsigned int found = 0;
 
-		filename = strdup(alpm_list_getdata(t));
+		if((filename = strdup(t->data)) == NULL) {
+			goto targcleanup;
+		}
+
+		/* trailing '/' causes lstat to dereference directory symlinks */
+		len = strlen(filename) - 1;
+		while(len > 0 && filename[len] == '/') {
+			filename[len--] = '\0';
+		}
 
 		if(lstat(filename, &buf) == -1) {
 			/*  if it is not a path but a program name, then check in PATH */
@@ -151,92 +156,104 @@ static int query_fileowner(alpm_list_t *targets)
 				if(search_path(&filename, &buf) == -1) {
 					pm_printf(ALPM_LOG_ERROR, _("failed to find '%s' in PATH: %s\n"),
 							filename, strerror(errno));
-					ret++;
-					free(filename);
-					continue;
+					goto targcleanup;
 				}
 			} else {
 				pm_printf(ALPM_LOG_ERROR, _("failed to read file '%s': %s\n"),
 						filename, strerror(errno));
-				ret++;
-				free(filename);
-				continue;
+				goto targcleanup;
 			}
 		}
 
-		if(S_ISDIR(buf.st_mode)) {
-			pm_printf(ALPM_LOG_ERROR,
-				_("cannot determine ownership of directory '%s'\n"), filename);
-			ret++;
-			free(filename);
-			continue;
-		}
-
-		bname = mbasename(filename);
-		dname = mdirname(filename);
-		/* for files in '/', there is no directory name to match */
-		if(strcmp(dname, "") == 0) {
-			rpath = NULL;
+		is_dir = S_ISDIR(buf.st_mode) ? 1 : 0;
+		if(is_dir) {
+			/* the entire filename is safe to resolve if we know it points to a dir,
+			 * and it's necessary in case it ends in . or .. */
+			dname = realpath(filename, NULL);
+			bname = mbasename(dname);
+			rpath = mdirname(dname);
 		} else {
-			rpath = resolve_path(dname);
-
-			if(!rpath) {
-				pm_printf(ALPM_LOG_ERROR, _("cannot determine real path for '%s': %s\n"),
-						filename, strerror(errno));
-				free(filename);
-				free(dname);
-				free(rpath);
-				ret++;
-				continue;
-			}
+			bname = mbasename(filename);
+			dname = mdirname(filename);
+			rpath = realpath(dname, NULL);
 		}
-		free(dname);
 
-		for(i = alpm_db_get_pkgcache(db_local); i && !found; i = alpm_list_next(i)) {
-			alpm_pkg_t *info = alpm_list_getdata(i);
+		bname_len = strlen(bname);
+		pbname_len = bname_len + is_dir;
+
+		if(!dname || !rpath) {
+			pm_printf(ALPM_LOG_ERROR, _("cannot determine real path for '%s': %s\n"),
+					filename, strerror(errno));
+			goto targcleanup;
+		}
+
+		for(i = packages; i && (!found || is_dir); i = alpm_list_next(i)) {
+			alpm_pkg_t *info = i->data;
 			alpm_filelist_t *filelist = alpm_pkg_get_files(info);
 			size_t j;
 
 			for(j = 0; j < filelist->count; j++) {
 				const alpm_file_t *file = filelist->files + j;
-				char *ppath, *pdname;
+				char *ppath;
 				const char *pkgfile = file->name;
+				size_t pkgfile_len = strlen(pkgfile);
 
-				/* avoid the costly resolve_path usage if the basenames don't match */
-				if(strcmp(mbasename(pkgfile), bname) != 0) {
+				/* make sure pkgfile and target are of the same type */
+				if(is_dir != (pkgfile[pkgfile_len - 1] == '/')) {
 					continue;
 				}
 
-				/* for files in '/', there is no directory name to match */
-				if(!rpath) {
-					print_query_fileowner(filename, info);
-					found = 1;
+				/* make sure pkgfile is long enough */
+				if(pkgfile_len < pbname_len) {
 					continue;
 				}
 
-				if(rootlen + 1 + strlen(pkgfile) > PATH_MAX) {
-					pm_printf(ALPM_LOG_ERROR, _("path too long: %s%s\n"), root, pkgfile);
+				/* make sure pbname_len takes us to the start of a path component */
+				if(pbname_len != pkgfile_len && pkgfile[pkgfile_len - pbname_len - 1] != '/') {
+					continue;
 				}
-				/* concatenate our file and the root path */
-				strcpy(path + rootlen, pkgfile);
 
-				pdname = mdirname(path);
-				ppath = resolve_path(pdname);
-				free(pdname);
+				/* compare basename with bname */
+				if(strncmp(pkgfile + pkgfile_len - pbname_len, bname, bname_len) != 0) {
+					continue;
+				}
 
-				if(ppath && strcmp(ppath, rpath) == 0) {
+				/* concatenate our dirname and the root path */
+				if(rootlen + 1 + pkgfile_len - pbname_len > PATH_MAX) {
+					path[rootlen] = '\0'; /* reset path for error message */
+					pm_printf(ALPM_LOG_ERROR, _("path too long: %s%s\n"), path, pkgfile);
+					continue;
+				}
+				strncpy(path + rootlen, pkgfile, pkgfile_len - pbname_len);
+				path[rootlen + pkgfile_len - pbname_len] = '\0';
+
+				ppath = realpath(path, NULL);
+				if(!ppath) {
+					pm_printf(ALPM_LOG_ERROR, _("cannot determine real path for '%s': %s\n"),
+							path, strerror(errno));
+					continue;
+				}
+
+				if(strcmp(ppath, rpath) == 0) {
 					print_query_fileowner(filename, info);
 					found = 1;
+					free(ppath);
+					break;
 				}
 				free(ppath);
 			}
 		}
 		if(!found) {
 			pm_printf(ALPM_LOG_ERROR, _("No package owns %s\n"), filename);
+		}
+
+targcleanup:
+		if(!found) {
 			ret++;
 		}
 		free(filename);
 		free(rpath);
+		free(dname);
 	}
 
 	return ret;
@@ -245,92 +262,40 @@ static int query_fileowner(alpm_list_t *targets)
 /* search the local database for a matching package */
 static int query_search(alpm_list_t *targets)
 {
-	alpm_list_t *i, *searchlist;
-	int freelist;
-	alpm_db_t *db_local = alpm_option_get_localdb(config->handle);
-
-	/* if we have a targets list, search for packages matching it */
-	if(targets) {
-		searchlist = alpm_db_search(db_local, targets);
-		freelist = 1;
-	} else {
-		searchlist = alpm_db_get_pkgcache(db_local);
-		freelist = 0;
-	}
-	if(searchlist == NULL) {
-		return 1;
-	}
-
-	for(i = searchlist; i; i = alpm_list_next(i)) {
-		alpm_list_t *grp;
-		alpm_pkg_t *pkg = alpm_list_getdata(i);
-
-		if(!config->quiet) {
-			printf("local/%s %s", alpm_pkg_get_name(pkg), alpm_pkg_get_version(pkg));
-		} else {
-			printf("%s", alpm_pkg_get_name(pkg));
-		}
-
-
-		if(!config->quiet) {
-			if((grp = alpm_pkg_get_groups(pkg)) != NULL) {
-				alpm_list_t *k;
-				printf(" (");
-				for(k = grp; k; k = alpm_list_next(k)) {
-					const char *group = alpm_list_getdata(k);
-					printf("%s", group);
-					if(alpm_list_next(k)) {
-						/* only print a spacer if there are more groups */
-						printf(" ");
-					}
-				}
-				printf(")");
-			}
-
-			/* we need a newline and initial indent first */
-			printf("\n    ");
-			indentprint(alpm_pkg_get_desc(pkg), 4);
-		}
-		printf("\n");
-	}
-
-	/* we only want to free if the list was a search list */
-	if(freelist) {
-		alpm_list_free(searchlist);
-	}
-	return 0;
+	alpm_db_t *db_local = alpm_get_localdb(config->handle);
+	return dump_pkg_search(db_local, targets, 0);
 }
 
 static int query_group(alpm_list_t *targets)
 {
 	alpm_list_t *i, *j;
-	char *grpname = NULL;
+	const char *grpname = NULL;
 	int ret = 0;
-	alpm_db_t *db_local = alpm_option_get_localdb(config->handle);
+	alpm_db_t *db_local = alpm_get_localdb(config->handle);
 
 	if(targets == NULL) {
 		for(j = alpm_db_get_groupcache(db_local); j; j = alpm_list_next(j)) {
-			alpm_group_t *grp = alpm_list_getdata(j);
+			alpm_group_t *grp = j->data;
 			const alpm_list_t *p;
 
 			for(p = grp->packages; p; p = alpm_list_next(p)) {
-				alpm_pkg_t *pkg = alpm_list_getdata(p);
+				alpm_pkg_t *pkg = p->data;
 				printf("%s %s\n", grp->name, alpm_pkg_get_name(pkg));
 			}
 		}
 	} else {
 		for(i = targets; i; i = alpm_list_next(i)) {
 			alpm_group_t *grp;
-			grpname = alpm_list_getdata(i);
-			grp = alpm_db_readgroup(db_local, grpname);
+			grpname = i->data;
+			grp = alpm_db_get_group(db_local, grpname);
 			if(grp) {
 				const alpm_list_t *p;
 				for(p = grp->packages; p; p = alpm_list_next(p)) {
 					if(!config->quiet) {
 						printf("%s %s\n", grpname,
-								alpm_pkg_get_name(alpm_list_getdata(p)));
+								alpm_pkg_get_name(p->data));
 					} else {
-						printf("%s\n", alpm_pkg_get_name(alpm_list_getdata(p)));
+						printf("%s\n", alpm_pkg_get_name(p->data));
 					}
 				}
 			} else {
@@ -342,25 +307,18 @@ static int query_group(alpm_list_t *targets)
 	return ret;
 }
 
-static int is_foreign(alpm_pkg_t *pkg)
+static unsigned short pkg_get_locality(alpm_pkg_t *pkg)
 {
 	const char *pkgname = alpm_pkg_get_name(pkg);
 	alpm_list_t *j;
-	alpm_list_t *sync_dbs = alpm_option_get_syncdbs(config->handle);
+	alpm_list_t *sync_dbs = alpm_get_syncdbs(config->handle);
 
-	int match = 0;
 	for(j = sync_dbs; j; j = alpm_list_next(j)) {
-		alpm_db_t *db = alpm_list_getdata(j);
-		alpm_pkg_t *findpkg = alpm_db_get_pkg(db, pkgname);
-		if(findpkg) {
-			match = 1;
-			break;
+		if(alpm_db_get_pkg(j->data, pkgname)) {
+			return PKG_LOCALITY_LOCAL;
 		}
 	}
-	if(match == 0) {
-		return 1;
-	}
-	return 0;
+	return PKG_LOCALITY_FOREIGN;
 }
 
 static int is_unrequired(alpm_pkg_t *pkg)
@@ -385,8 +343,8 @@ static int filter(alpm_pkg_t *pkg)
 			alpm_pkg_get_reason(pkg) != ALPM_PKG_REASON_DEPEND) {
 		return 0;
 	}
-	/* check if this pkg isn't in a sync DB */
-	if(config->op_q_foreign && !is_foreign(pkg)) {
+	/* check if this pkg is or isn't in a sync DB */
+	if(config->op_q_locality && config->op_q_locality & pkg_get_locality(pkg)) {
 		return 0;
 	}
 	/* check if this pkg is unrequired */
@@ -395,66 +353,12 @@ static int filter(alpm_pkg_t *pkg)
 	}
 	/* check if this pkg is outdated */
 	if(config->op_q_upgrade && (alpm_sync_newversion(pkg,
-					alpm_option_get_syncdbs(config->handle)) == NULL)) {
+					alpm_get_syncdbs(config->handle)) == NULL)) {
 		return 0;
 	}
 	return 1;
 }
 
-/* Loop through the packages. For each package,
- * loop through files to check if they exist. */
-static int check(alpm_pkg_t *pkg)
-{
-	const char *root, *pkgname;
-	size_t errors = 0;
-	size_t rootlen;
-	char f[PATH_MAX];
-	alpm_filelist_t *filelist;
-	size_t i;
-
-	root = alpm_option_get_root(config->handle);
-	rootlen = strlen(root);
-	if(rootlen + 1 > PATH_MAX) {
-		/* we are in trouble here */
-		pm_printf(ALPM_LOG_ERROR, _("path too long: %s%s\n"), root, "");
-		return 1;
-	}
-	strcpy(f, root);
-
-	pkgname = alpm_pkg_get_name(pkg);
-	filelist = alpm_pkg_get_files(pkg);
-	for(i = 0; i < filelist->count; i++) {
-		const alpm_file_t *file = filelist->files + i;
-		struct stat st;
-		const char *path = file->name;
-
-		if(rootlen + 1 + strlen(path) > PATH_MAX) {
-			pm_printf(ALPM_LOG_WARNING, _("path too long: %s%s\n"), root, path);
-			continue;
-		}
-		strcpy(f + rootlen, path);
-		/* use lstat to prevent errors from symlinks */
-		if(lstat(f, &st) != 0) {
-			if(config->quiet) {
-				printf("%s %s\n", pkgname, f);
-			} else {
-				pm_printf(ALPM_LOG_WARNING, "%s: %s (%s)\n",
-						pkgname, f, strerror(errno));
-			}
-			errors++;
-		}
-	}
-
-	if(!config->quiet) {
-		printf(_n("%s: %jd total file, ", "%s: %jd total files, ",
-					(unsigned long)filelist->count), pkgname, (intmax_t)filelist->count);
-		printf(_n("%jd missing file\n", "%jd missing files\n",
-					(unsigned long)errors), (intmax_t)errors);
-	}
-
-	return (errors != 0 ? 1 : 0);
-}
-
 static int display(alpm_pkg_t *pkg)
 {
 	int ret = 0;
@@ -473,12 +377,18 @@ static int display(alpm_pkg_t *pkg)
 		dump_pkg_changelog(pkg);
 	}
 	if(config->op_q_check) {
-		ret = check(pkg);
+		if(config->op_q_check == 1) {
+			ret = check_pkg_fast(pkg);
+		} else {
+			ret = check_pkg_full(pkg);
+		}
 	}
 	if(!config->op_q_info && !config->op_q_list
 			&& !config->op_q_changelog && !config->op_q_check) {
 		if(!config->quiet) {
-			printf("%s %s\n", alpm_pkg_get_name(pkg), alpm_pkg_get_version(pkg));
+			const colstr_t *colstr = &config->colstr;
+			printf("%s%s %s%s%s\n", colstr->title, alpm_pkg_get_name(pkg),
+					colstr->version, alpm_pkg_get_version(pkg), colstr->nocolor);
 		} else {
 			printf("%s\n", alpm_pkg_get_name(pkg));
 		}
@@ -508,13 +418,13 @@ int pacman_query(alpm_list_t *targets)
 		return ret;
 	}
 
-	if(config->op_q_foreign || config->op_q_upgrade) {
+	if(config->op_q_locality || config->op_q_upgrade) {
 		if(check_syncdbs(1, 1)) {
 			return 1;
 		}
 	}
 
-	db_local = alpm_option_get_localdb(config->handle);
+	db_local = alpm_get_localdb(config->handle);
 
 	/* operations on all packages in the local DB
 	 * valid: no-op (plain -Q), list, info, check
@@ -526,7 +436,7 @@ int pacman_query(alpm_list_t *targets)
 		}
 
 		for(i = alpm_db_get_pkgcache(db_local); i; i = alpm_list_next(i)) {
-			pkg = alpm_list_getdata(i);
+			pkg = i->data;
 			if(filter(pkg)) {
 				int value = display(pkg);
 				if(value != 0) {
@@ -552,7 +462,12 @@ int pacman_query(alpm_list_t *targets)
 	/* operations on named packages in the local DB
 	 * valid: no-op (plain -Q), list, info, check */
 	for(i = targets; i; i = alpm_list_next(i)) {
-		char *strname = alpm_list_getdata(i);
+		const char *strname = i->data;
+
+		/* strip leading part of "local/pkgname" */
+		if(strncmp(strname, LOCAL_PREFIX, strlen(LOCAL_PREFIX)) == 0) {
+			strname += strlen(LOCAL_PREFIX);
+		}
 
 		if(config->op_q_isfile) {
 			alpm_pkg_load(config->handle, strname, 1, 0, &pkg);
