@@ -1,7 +1,7 @@
 /*
  *  upgrade.c
  *
- *  Copyright (c) 2006-2011 Pacman Development Team <pacman-dev@archlinux.org>
+ *  Copyright (c) 2006-2013 Pacman Development Team <pacman-dev@archlinux.org>
  *  Copyright (c) 2002-2006 by Judd Vinet <jvinet@zeroflux.org>
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -18,8 +18,6 @@
  *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-#include "config.h"
-
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
@@ -41,8 +39,8 @@
  */
 int pacman_upgrade(alpm_list_t *targets)
 {
-	alpm_list_t *i;
-	alpm_siglevel_t level = alpm_option_get_default_siglevel(config->handle);
+	int retval = 0;
+	alpm_list_t *i, *remote = NULL;
 
 	if(targets == NULL) {
 		pm_printf(ALPM_LOG_ERROR, _("no targets specified (use -h for help)\n"));
@@ -52,17 +50,28 @@ int pacman_upgrade(alpm_list_t *targets)
 	/* Check for URL targets and process them
 	 */
 	for(i = targets; i; i = alpm_list_next(i)) {
+		int *r = malloc(sizeof(int));
+
 		if(strstr(i->data, "://")) {
 			char *str = alpm_fetch_pkgurl(config->handle, i->data);
 			if(str == NULL) {
 				pm_printf(ALPM_LOG_ERROR, "'%s': %s\n",
 						(char *)i->data, alpm_strerror(alpm_errno(config->handle)));
-				return 1;
+				retval = 1;
 			} else {
 				free(i->data);
 				i->data = str;
+				*r = 1;
 			}
+		} else {
+			*r = 0;
 		}
+
+		remote = alpm_list_add(remote, r);
+	}
+
+	if(retval) {
+		return retval;
 	}
 
 	/* Step 1: create a new transaction */
@@ -72,26 +81,40 @@ int pacman_upgrade(alpm_list_t *targets)
 
 	printf(_("loading packages...\n"));
 	/* add targets to the created transaction */
-	for(i = targets; i; i = alpm_list_next(i)) {
-		char *targ = alpm_list_getdata(i);
+	for(i = targets; i; i = alpm_list_next(i), remote = alpm_list_next(remote)) {
+		const char *targ = i->data;
 		alpm_pkg_t *pkg;
+		alpm_siglevel_t level;
+
+		if(*(int *)remote->data) {
+			level = alpm_option_get_remote_file_siglevel(config->handle);
+		} else {
+			level = alpm_option_get_local_file_siglevel(config->handle);
+		}
 
 		if(alpm_pkg_load(config->handle, targ, 1, level, &pkg) != 0) {
 			pm_printf(ALPM_LOG_ERROR, "'%s': %s\n",
 					targ, alpm_strerror(alpm_errno(config->handle)));
-			trans_release();
-			return 1;
+			retval = 1;
+			continue;
 		}
 		if(alpm_add_pkg(config->handle, pkg) == -1) {
 			pm_printf(ALPM_LOG_ERROR, "'%s': %s\n",
 					targ, alpm_strerror(alpm_errno(config->handle)));
 			alpm_pkg_free(pkg);
-			trans_release();
-			return 1;
+			retval = 1;
+			continue;
 		}
 		config->explicit_adds = alpm_list_add(config->explicit_adds, pkg);
 	}
 
+	FREELIST(remote);
+
+	if(retval) {
+		trans_release();
+		return retval;
+	}
+
 	/* now that targets are resolved, we can hand it all off to the sync code */
 	return sync_prepare_execute();
 }
