@@ -1,7 +1,7 @@
 /*
  *  util.h
  *
- *  Copyright (c) 2006-2012 Pacman Development Team <pacman-dev@archlinux.org>
+ *  Copyright (c) 2006-2013 Pacman Development Team <pacman-dev@archlinux.org>
  *  Copyright (c) 2002-2006 by Judd Vinet <jvinet@zeroflux.org>
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -26,6 +26,8 @@
 
 #include <alpm_list.h>
 
+#include "util-common.h"
+
 #ifdef ENABLE_NLS
 #include <libintl.h> /* here so it doesn't need to be included elsewhere */
 /* define _() as shortcut for gettext() */
@@ -36,9 +38,6 @@
 #define _n(str1, str2, ct) (ct == 1 ? str1 : str2)
 #endif
 
-/* update speed for the fill_progress based functions */
-#define UPDATE_SPEED_SEC 0.2f
-
 typedef struct _pm_target_t {
 	alpm_pkg_t *remove;
 	alpm_pkg_t *install;
@@ -50,21 +49,21 @@ int trans_init(alpm_transflag_t flags, int check_valid);
 int trans_release(void);
 int needs_root(void);
 int check_syncdbs(size_t need_repos, int check_valid);
-unsigned short getcols(void);
+unsigned short getcols(int fd);
 int rmrf(const char *path);
-const char *mbasename(const char *path);
-char *mdirname(const char *path);
-void indentprint(const char *str, size_t indent);
-char *strtrim(char *str);
+void indentprint(const char *str, unsigned short indent, unsigned short cols);
+size_t strtrim(char *str);
 char *strreplace(const char *str, const char *needle, const char *replace);
 alpm_list_t *strsplit(const char *str, const char splitchar);
-void string_display(const char *title, const char *string);
+void string_display(const char *title, const char *string, unsigned short cols);
 double humanize_size(off_t bytes, const char target_unit, int precision,
 		const char **label);
-int table_display(const char *title, const alpm_list_t *header, const alpm_list_t *rows);
-void list_display(const char *title, const alpm_list_t *list);
-void list_display_linebreak(const char *title, const alpm_list_t *list);
-void signature_display(const char *title, alpm_siglist_t *siglist);
+void list_display(const char *title, const alpm_list_t *list,
+		unsigned short maxcols);
+void list_display_linebreak(const char *title, const alpm_list_t *list,
+		unsigned short maxcols);
+void signature_display(const char *title, alpm_siglist_t *siglist,
+		unsigned short maxcols);
 void display_targets(void);
 int str_cmp(const void *s1, const void *s2);
 void display_new_optdepends(alpm_pkg_t *oldpkg, alpm_pkg_t *newpkg);
@@ -73,18 +72,15 @@ void print_packages(const alpm_list_t *packages);
 void select_display(const alpm_list_t *pkglist);
 int select_question(int count);
 int multiselect_question(char *array, int count);
-int yesno(char *fmt, ...);
-int noyes(char *fmt, ...);
+int colon_printf(const char *format, ...) __attribute__((format(printf, 1, 2)));
+int yesno(const char *format, ...) __attribute__((format(printf, 1, 2)));
+int noyes(const char *format, ...) __attribute__((format(printf, 1, 2)));
 
 int pm_printf(alpm_loglevel_t level, const char *format, ...) __attribute__((format(printf,2,3)));
-int pm_asprintf(char **string, const char *format, ...);
+int pm_asprintf(char **string, const char *format, ...) __attribute__((format(printf,2,3)));
 int pm_vfprintf(FILE *stream, alpm_loglevel_t level, const char *format, va_list args) __attribute__((format(printf,3,0)));
 int pm_vasprintf(char **string, alpm_loglevel_t level, const char *format, va_list args) __attribute__((format(printf,3,0)));
 
-#ifndef HAVE_STRNDUP
-char *strndup(const char *s, size_t n);
-#endif
-
 #endif /* _PM_UTIL_H */
 
 /* vim: set ts=2 sw=2 noet: */
