@@ -17,8 +17,6 @@
  *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-#include "config.h"
-
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -30,7 +28,8 @@
 
 alpm_handle_t *handle = NULL;
 
-static void cleanup(int signum) {
+static void cleanup(int signum)
+{
 	if(handle && alpm_release(handle) == -1) {
 		fprintf(stderr, "error releasing alpm\n");
 	}
@@ -38,13 +37,14 @@ static void cleanup(int signum) {
 	exit(signum);
 }
 
+__attribute__((format(printf, 2, 0)))
 static void output_cb(alpm_loglevel_t level, const char *fmt, va_list args)
 {
 	if(strlen(fmt)) {
 		switch(level) {
 			case ALPM_LOG_ERROR: printf("error: "); break;
 			case ALPM_LOG_WARNING: printf("warning: "); break;
-			//case ALPM_LOG_DEBUG: printf("debug: "); break;
+			/* case ALPM_LOG_DEBUG: printf("debug: "); break; */
 			default: return;
 		}
 		vprintf(fmt, args);
@@ -56,24 +56,25 @@ static void checkpkgs(alpm_list_t *pkglist)
 {
 	alpm_list_t *i, *j;
 	for(i = pkglist; i; i = alpm_list_next(i)) {
-		alpm_pkg_t *pkg = alpm_list_getdata(i);
+		alpm_pkg_t *pkg = i->data;
 		alpm_list_t *unused = alpm_pkg_unused_deltas(pkg);
 		for(j = unused; j; j = alpm_list_next(j)) {
-			char *delta = alpm_list_getdata(j);
+			const char *delta = j->data;
 			printf("%s\n", delta);
 		}
 		alpm_list_free(unused);
 	}
 }
 
-static void checkdbs(alpm_list_t *dbnames) {
+static void checkdbs(alpm_list_t *dbnames)
+{
 	alpm_db_t *db = NULL;
 	alpm_list_t *i;
 	const alpm_siglevel_t level = ALPM_SIG_DATABASE | ALPM_SIG_DATABASE_OPTIONAL;
 
 	for(i = dbnames; i; i = alpm_list_next(i)) {
-		const char *dbname = alpm_list_getdata(i);
-		db = alpm_db_register_sync(handle, dbname, level);
+		const char *dbname = i->data;
+		db = alpm_register_syncdb(handle, dbname, level);
 		if(db == NULL) {
 			fprintf(stderr, "error: could not register sync database '%s' (%s)\n",
 					dbname, alpm_strerror(alpm_errno(handle)));
@@ -84,7 +85,8 @@ static void checkdbs(alpm_list_t *dbnames) {
 
 }
 
-static void usage(void) {
+static void usage(void)
+{
 	fprintf(stderr, "usage:\n");
 	fprintf(stderr,
 			"\t%s [-b <pacman db>] core extra ... : check the listed sync databases\n", BASENAME);
@@ -94,7 +96,7 @@ static void usage(void) {
 int main(int argc, char *argv[])
 {
 	const char *dbpath = DBPATH;
-	enum _alpm_errno_t err;
+	alpm_errno_t err;
 	int a = 1;
 	alpm_list_t *dbnames = NULL;
 
