@@ -17,37 +17,61 @@
  *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-#include "config.h"
-
 #include <ctype.h>
 #include <getopt.h>
 #include <stdio.h>
 #include <string.h>
-
+#include <locale.h>
 #include <alpm.h>
 #include <alpm_list.h>
 
+#ifdef HAVE_LANGINFO_H
+#include <langinfo.h>
+#endif
+
 #define LINE_MAX     512
 
+typedef struct tdepth {
+	struct tdepth *prev;
+	struct tdepth *next;
+	int level;
+} tdepth;
+
 /* output */
 struct graph_style {
 	const char *provides;
-	const char *tip1;
-	const char *tip2;
+	const char *tip;
+	const char *last;
+	const char *limb;
 	int indent;
 };
 
+#define UTF_V   "\342\224\202"  /* U+2502, Vertical line drawing char */
+#define UTF_VR  "\342\224\234"  /* U+251C, Vertical and right */
+#define UTF_H   "\342\224\200"  /* U+2500, Horizontal */
+#define UTF_UR  "\342\224\224"  /* U+2514, Up and right */
+
+static struct graph_style graph_utf8 = {
+	" provides",
+	UTF_VR UTF_H,
+	UTF_UR UTF_H,
+	UTF_V " ",
+	2
+};
+
 static struct graph_style graph_default = {
 	" provides",
-	"|--",
-	"+--",
-	3
+	"|-",
+	"`-",
+	"|",
+	2
 };
 
 static struct graph_style graph_linear = {
 	"",
 	"",
 	"",
+	"",
 	0
 };
 
@@ -88,7 +112,7 @@ alpm_list_t *provisions = NULL;
 
 /* options */
 struct color_choices *color = &no_color;
-struct graph_style *style = &graph_default;
+struct graph_style *style = &graph_utf8;
 int graphviz = 0;
 int max_depth = -1;
 int reverse = 0;
@@ -119,13 +143,13 @@ char *strndup(const char *s, size_t n)
 }
 #endif
 
-static char *strtrim(char *str)
+static size_t strtrim(char *str)
 {
-	char *pch = str;
+	char *end, *pch = str;
 
 	if(str == NULL || *str == '\0') {
 		/* string is empty, so we're done. */
-		return str;
+		return 0;
 	}
 
 	while(isspace((unsigned char)*pch)) {
@@ -142,21 +166,22 @@ static char *strtrim(char *str)
 
 	/* check if there wasn't anything but whitespace in the string. */
 	if(*str == '\0') {
-		return str;
+		return 0;
 	}
 
-	pch = (str + (strlen(str) - 1));
-	while(isspace((unsigned char)*pch)) {
-		pch--;
+	end = (str + strlen(str) - 1);
+	while(isspace((unsigned char)*end)) {
+		end--;
 	}
-	*++pch = '\0';
+	*++end = '\0';
 
-	return str;
+	return end - pch;
 }
 
-static int register_syncs(void) {
+static int register_syncs(void)
+{
 	FILE *fp;
-	char *ptr, *section = NULL;
+	char *section = NULL;
 	char line[LINE_MAX];
 	const alpm_siglevel_t level = ALPM_SIG_DATABASE | ALPM_SIG_DATABASE_OPTIONAL;
 
@@ -167,23 +192,26 @@ static int register_syncs(void) {
 	}
 
 	while(fgets(line, LINE_MAX, fp)) {
+		size_t linelen;
+		char *ptr;
+
 		/* ignore whole line and end of line comments */
 		if((ptr = strchr(line, '#'))) {
 			*ptr = '\0';
 		}
 
-		strtrim(line);
+		linelen = strtrim(line);
 
-		if(strlen(line) == 0) {
+		if(linelen == 0) {
 			continue;
 		}
 
-		if(line[0] == '[' && line[strlen(line) - 1] == ']') {
+		if(line[0] == '[' && line[linelen - 1] == ']') {
 			free(section);
-			section = strndup(&line[1], strlen(line) - 2);
+			section = strndup(&line[1], linelen - 2);
 
 			if(section && strcmp(section, "options") != 0) {
-				alpm_db_register_sync(handle, section, level);
+				alpm_register_syncdb(handle, section, level);
 			}
 		}
 	}
@@ -200,6 +228,7 @@ static int parse_options(int argc, char *argv[])
 	char *endptr = NULL;
 
 	static const struct option opts[] = {
+		{"ascii",   no_argument,          0, 'a'},
 		{"dbpath",  required_argument,    0, 'b'},
 		{"color",   no_argument,          0, 'c'},
 		{"depth",   required_argument,    0, 'd'},
@@ -207,19 +236,31 @@ static int parse_options(int argc, char *argv[])
 		{"help",    no_argument,          0, 'h'},
 		{"linear",  no_argument,          0, 'l'},
 		{"reverse", no_argument,          0, 'r'},
-		{"sync",    no_argument,          0, 'S'},
+		{"sync",    no_argument,          0, 's'},
 		{"unique",  no_argument,          0, 'u'},
 
 		{"config",  required_argument,    0, OP_CONFIG},
 		{0, 0, 0, 0}
 	};
 
-	while((opt = getopt_long(argc, argv, "b:cd:ghlrsu", opts, &option_index))) {
+#ifdef HAVE_LANGINFO_H
+	setlocale(LC_ALL, "");
+	if(strcmp(nl_langinfo(CODESET), "UTF-8") == 0) {
+		style = &graph_utf8;
+	}
+#else
+	style = &graph_default;
+#endif
+
+	while((opt = getopt_long(argc, argv, "ab:cd:ghlrsu", opts, &option_index))) {
 		if(opt < 0) {
 			break;
 		}
 
 		switch(opt) {
+			case 'a':
+				style = &graph_default;
+				break;
 			case OP_CONFIG:
 				configfile = optarg;
 				break;
@@ -271,6 +312,7 @@ static void usage(void)
 {
 	fprintf(stderr, "pactree v" PACKAGE_VERSION "\n"
 			"Usage: pactree [options] PACKAGE\n\n"
+			"  -a, --ascii          use ascii characters for tree formatting\n"
 			"  -b, --dbpath <path>  set an alternate database location\n"
 			"  -c, --color          colorize output\n"
 			"  -d, --depth <#>      limit the depth of recursion\n"
@@ -291,28 +333,43 @@ static void cleanup(void)
 }
 
 /* pkg provides provision */
-static void print_text(const char *pkg, const char *provision, int depth)
+static void print_text(const char *pkg, const char *provision,
+		tdepth *depth, int last)
 {
-	int indent_sz = (depth + 1) * style->indent;
-
+	const char* tip = "";
+	int level = 1;
 	if(!pkg && !provision) {
 		/* not much we can do */
 		return;
 	}
 
+	if(depth->level > 0) {
+		tip = last ? style->last : style->tip;
+
+		/* print limbs */
+		while(depth->prev) {
+			depth = depth->prev;
+		}
+		printf("%s", color->branch1);
+		while(depth->next) {
+			printf("%*s%-*s", style->indent * (depth->level - level), "",
+					style->indent, style->limb);
+			level = depth->level + 1;
+			depth = depth->next;
+		}
+		printf("%*s", style->indent * (depth->level - level), "");
+	}
+
+	/* print tip */
 	if(!pkg && provision) {
-		/* we failed to resolve provision */
-		printf("%s%*s%s%s%s [unresolvable]%s\n", color->branch1, indent_sz,
-				style->tip1, color->leaf1, provision, color->branch1, color->off);
+		printf("%s%s%s%s [unresolvable]%s\n", tip,  color->leaf1,
+				provision, color->branch1, color->off);
 	} else if(provision && strcmp(pkg, provision) != 0) {
-		/* pkg provides provision */
-		printf("%s%*s%s%s%s%s %s%s%s\n", color->branch2, indent_sz, style->tip2,
-				color->leaf1, pkg, color->leaf2, style->provides, color->leaf1, provision,
+		printf("%s%s%s%s%s %s%s%s\n", tip, color->leaf1, pkg,
+				color->leaf2, style->provides, color->leaf1, provision,
 				color->off);
 	} else {
-		/* pkg is a normal package */
-		printf("%s%*s%s%s%s\n", color->branch1, indent_sz, style->tip1, color->leaf1,
-				pkg, color->off);
+		printf("%s%s%s%s\n", tip, color->leaf1, pkg, color->off);
 	}
 }
 
@@ -330,12 +387,13 @@ static void print_graph(const char *parentname, const char *pkgname, const char
 }
 
 /* parent depends on dep which is satisfied by pkg */
-static void print(const char *parentname, const char *pkgname, const char *depname, int depth)
+static void print(const char *parentname, const char *pkgname,
+		const char *depname, tdepth *depth, int last)
 {
 	if(graphviz) {
 		print_graph(parentname, pkgname, depname);
 	} else {
-		print_text(pkgname, depname, depth);
+		print_text(pkgname, depname, depth, last);
 	}
 }
 
@@ -346,7 +404,12 @@ static void print_start(const char *pkgname, const char *provname)
 				"node [style=filled, color=green];\n"
 				" \"START\" -> \"%s\";\n", pkgname);
 	} else {
-		print_text(pkgname, provname, 0);
+		tdepth d = {
+			NULL,
+			NULL,
+			0
+		};
+		print_text(pkgname, provname, &d, 0);
 	}
 }
 
@@ -358,92 +421,83 @@ static void print_end(void)
 	}
 }
 
-static alpm_pkg_t *get_pkg_from_dbs(alpm_list_t *dbs, const char *needle) {
-	alpm_list_t *i;
-	alpm_pkg_t *ret;
-
-	for(i = dbs; i; i = alpm_list_next(i)) {
-		ret = alpm_db_get_pkg(alpm_list_getdata(i), needle);
-		if(ret) {
-			return ret;
-		}
-	}
-	return NULL;
-}
-
-/**
- * walk dependencies in reverse, showing packages which require the target
- */
-static void walk_reverse_deps(alpm_list_t *dblist, alpm_pkg_t *pkg, int depth)
+static alpm_list_t *get_pkg_dep_names(alpm_pkg_t *pkg)
 {
-	alpm_list_t *required_by, *i;
-
-	if(!pkg || ((max_depth >= 0) && (depth == max_depth + 1))) {
-		return;
+	alpm_list_t *i, *names = NULL;
+	for(i = alpm_pkg_get_depends(pkg); i; i = alpm_list_next(i)) {
+		alpm_depend_t *d = i->data;
+		names = alpm_list_add(names, d->name);
 	}
-
-	walked = alpm_list_add(walked, (void *)alpm_pkg_get_name(pkg));
-	required_by = alpm_pkg_compute_requiredby(pkg);
-
-	for(i = required_by; i; i = alpm_list_next(i)) {
-		const char *pkgname = alpm_list_getdata(i);
-
-		if(alpm_list_find_str(walked, pkgname)) {
-			/* if we've already seen this package, don't print in "unique" output
-			 * and don't recurse */
-			if(!unique) {
-				print(alpm_pkg_get_name(pkg), pkgname, NULL, depth);
-			}
-		} else {
-			print(alpm_pkg_get_name(pkg), pkgname, NULL, depth);
-			walk_reverse_deps(dblist, get_pkg_from_dbs(dblist, pkgname), depth + 1);
-		}
-	}
-
-	FREELIST(required_by);
+	return names;
 }
 
 /**
  * walk dependencies, showing dependencies of the target
  */
-static void walk_deps(alpm_list_t *dblist, alpm_pkg_t *pkg, int depth)
+static void walk_deps(alpm_list_t *dblist, alpm_pkg_t *pkg, tdepth *depth, int rev)
 {
-	alpm_list_t *i;
+	alpm_list_t *deps, *i;
 
-	if((max_depth >= 0) && (depth == max_depth + 1)) {
+	if(!pkg || ((max_depth >= 0) && (depth->level > max_depth))) {
 		return;
 	}
 
 	walked = alpm_list_add(walked, (void *)alpm_pkg_get_name(pkg));
 
-	for(i = alpm_pkg_get_depends(pkg); i; i = alpm_list_next(i)) {
-		alpm_depend_t *depend = alpm_list_getdata(i);
-		alpm_pkg_t *provider = alpm_find_dbs_satisfier(handle, dblist, depend->name);
+	if(rev) {
+		deps = alpm_pkg_compute_requiredby(pkg);
+	} else {
+		deps = get_pkg_dep_names(pkg);
+	}
 
-		if(provider) {
-			const char *provname = alpm_pkg_get_name(provider);
+	for(i = deps; i; i = alpm_list_next(i)) {
+		const char *pkgname = i->data;
+		int last = alpm_list_next(i) ? 0 : 1;
 
-			if(alpm_list_find_str(walked, provname)) {
-				/* if we've already seen this package, don't print in "unique" output
-				 * and don't recurse */
-				if(!unique) {
-					print(alpm_pkg_get_name(pkg), provname, depend->name, depth);
-				}
-			} else {
-				print(alpm_pkg_get_name(pkg), provname, depend->name, depth);
-				walk_deps(dblist, provider, depth + 1);
+		alpm_pkg_t *dep_pkg = alpm_find_dbs_satisfier(handle, dblist, pkgname);
+
+		if(alpm_list_find_str(walked, dep_pkg ? alpm_pkg_get_name(dep_pkg) : pkgname)) {
+			/* if we've already seen this package, don't print in "unique" output
+			 * and don't recurse */
+			if(!unique) {
+				print(alpm_pkg_get_name(pkg), alpm_pkg_get_name(dep_pkg), pkgname, depth, last);
 			}
 		} else {
-			/* unresolvable package */
-			print(alpm_pkg_get_name(pkg), NULL, depend->name, depth);
+			print(alpm_pkg_get_name(pkg), alpm_pkg_get_name(dep_pkg), pkgname, depth, last);
+			if(dep_pkg) {
+				tdepth d = {
+					depth,
+					NULL,
+					depth->level + 1
+				};
+				depth->next = &d;
+				/* last dep, cut off the limb here */
+				if(last) {
+					if(depth->prev) {
+						depth->prev->next = &d;
+						d.prev = depth->prev;
+						depth = &d;
+					} else {
+						d.prev = NULL;
+					}
+				}
+				walk_deps(dblist, dep_pkg, &d, rev);
+				depth->next = NULL;
+			}
 		}
 	}
+
+	if(rev) {
+		FREELIST(deps);
+	} else {
+		alpm_list_free(deps);
+	}
 }
 
 int main(int argc, char *argv[])
 {
 	int freelist = 0, ret = 0;
-	enum _alpm_errno_t err;
+	alpm_errno_t err;
 	const char *target_name;
 	alpm_pkg_t *pkg;
 	alpm_list_t *dblist = NULL;
@@ -467,9 +521,9 @@ int main(int argc, char *argv[])
 			ret = 1;
 			goto finish;
 		}
-		dblist = alpm_option_get_syncdbs(handle);
+		dblist = alpm_get_syncdbs(handle);
 	} else {
-		dblist = alpm_list_add(dblist, alpm_option_get_localdb(handle));
+		dblist = alpm_list_add(dblist, alpm_get_localdb(handle));
 		freelist = 1;
 	}
 
@@ -485,11 +539,12 @@ int main(int argc, char *argv[])
 
 	print_start(alpm_pkg_get_name(pkg), target_name);
 
-	if(reverse) {
-		walk_reverse_deps(dblist, pkg, 1);
-	} else {
-		walk_deps(dblist, pkg, 1);
-	}
+	tdepth d = {
+		NULL,
+		NULL,
+		1
+	};
+	walk_deps(dblist, pkg, &d, reverse);
 
 	print_end();
 
