@@ -1,4 +1,4 @@
-self.description = "Query info on a package"
+self.description = "Query info on a package (old date)"
 
 p = pmpkg("foobar")
 p.files = ["bin/foobar"]
@@ -18,4 +18,4 @@ self.args = "-Qi %s" % p.name
 self.addrule("PACMAN_RETCODE=0")
 self.addrule("PACMAN_OUTPUT=^Name.*%s" % p.name)
 self.addrule("PACMAN_OUTPUT=^Description.*%s" % p.desc)
-self.addrule("PACMAN_OUTPUT=^Build Date.*2007")
+self.addrule("PACMAN_OUTPUT=^Build Date.* 2007")
