@@ -36,7 +36,7 @@ pass() {
 fail() {
 	echo "test: ver1: $1 ver2: $2 ret: $3 expected: $4"
 	echo "  ==> FAILURE"
-	failure=$(expr $failure + 1)
+	((failure++))
 }
 
 # args:
@@ -45,29 +45,30 @@ runtest() {
 	# run the test
 	ret=$($bin $1 $2)
 	func='pass'
-	[ $ret -eq $3 ] || func='fail'
+	[[ -n $ret && $ret -eq $3 ]] || func='fail'
 	$func $1 $2 $ret $3
-	total=$(expr $total + 1)
+	((total++))
 	# and run its mirror case just to be sure
 	reverse=0
-	[ $3 -eq 1 ] && reverse=-1
-	[ $3 -eq -1 ] && reverse=1
+	[[ $3 -eq 1 ]] && reverse=-1
+	[[ $3 -eq -1 ]] && reverse=1
 	ret=$($bin $2 $1)
 	func='pass'
-	[ $ret -eq $reverse ] || func='fail'
+	[[ -n $ret && $ret -eq $reverse ]] || func='fail'
 	$func $2 $1 $ret $reverse
-	total=$(expr $total + 1)
+	((total++))
 }
 
 # use first arg as our binary if specified
-[ -n "$1" ] && bin="$1"
+[[ -n "$1" ]] && bin="$1"
 
-if [ ! $(type -p "$bin") ]; then
+if ! type -p "$bin"; then
 	echo "vercmp binary ($bin) could not be located"
+	echo
 	exit 1
 fi
 
-echo "Beginning vercmp tests"
+echo "Running vercmp tests..."
 
 # BEGIN TESTS
 
@@ -118,6 +119,12 @@ runtest 1.5.1    1.5.b   1
 runtest 1.5.b-1  1.5.b   0
 runtest 1.5-1    1.5.b  -1
 
+# same/similar content, differing separators
+runtest 2.0    2_0     0
+runtest 2.0_a  2_0.a   0
+runtest 2.0a   2.0.a  -1
+runtest 2___a  2_a     1
+
 # epoch included version comparisons
 runtest 0:1.0    0:1.0   0
 runtest 0:1.0    0:1.1  -1
@@ -139,11 +146,12 @@ runtest 1:1.1    1.1   1
 
 #END TESTS
 
-echo
-if [ $failure -eq 0 ]; then
+if [[ $failure -eq 0 ]]; then
 	echo "All $total tests successful"
+	echo
 	exit 0
 fi
 
 echo "$failure of $total tests failed"
+echo
 exit 1
